@@ -1,6 +1,8 @@
 /* 
    neon SSL/TLS support using OpenSSL
-   Copyright (C) 2002-2021, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2007, Joe Orton <joe@manyfish.co.uk>
+   Portions are:
+   Copyright (C) 1999-2000 Tommi Komulainen <Tommi.Komulainen@iki.fi>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -34,21 +36,17 @@
 #include <openssl/pkcs12.h>
 #include <openssl/x509v3.h>
 #include <openssl/rand.h>
-#include <openssl/opensslv.h>
-#include <openssl/evp.h>
 
 #ifdef NE_HAVE_TS_SSL
 #include <stdlib.h> /* for abort() */
-#ifndef _WIN32
 #include <pthread.h>
 #endif
-#endif
 
 #include "ne_ssl.h"
 #include "ne_string.h"
 #include "ne_session.h"
 #include "ne_internal.h"
-#include "ne_md5.h"
+
 #include "ne_private.h"
 #include "ne_privssl.h"
 
@@ -67,17 +65,6 @@ typedef unsigned char ne_d2i_uchar;
 typedef const unsigned char ne_d2i_uchar;
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-#define X509_get0_notBefore X509_get_notBefore
-#define X509_get0_notAfter X509_get_notAfter
-#define X509_up_ref(x) x->references++
-#define EVP_PKEY_up_ref(x) x->references++
-#define EVP_MD_CTX_new() ne_calloc(sizeof(EVP_MD_CTX))
-#define EVP_MD_CTX_free(ctx) ne_free(ctx)
-#define EVP_MD_CTX_reset EVP_MD_CTX_cleanup
-#define EVP_PKEY_get0_RSA(evp) (evp->pkey.rsa)
-#endif
-
 struct ne_ssl_dname_s {
     X509_NAME *dn;
 };
@@ -97,8 +84,6 @@ struct ne_ssl_client_cert_s {
     char *friendly_name;
 };
 
-#define NE_SSL_UNHANDLED (0x20) /* failure bit for unhandled case. */
-
 /* Append an ASN.1 DirectoryString STR to buffer BUF as UTF-8.
  * Returns zero on success or non-zero on error. */
 static int append_dirstring(ne_buffer *buf, ASN1_STRING *str)
@@ -107,16 +92,10 @@ static int append_dirstring(ne_buffer *buf, ASN1_STRING *str)
     int len;
 
     switch (str->type) {
+    case V_ASN1_UTF8STRING:
     case V_ASN1_IA5STRING: /* definitely ASCII */
     case V_ASN1_VISIBLESTRING: /* probably ASCII */
     case V_ASN1_PRINTABLESTRING: /* subset of ASCII */
-        ne_buffer_qappend(buf, str->data, str->length);
-        break;
-    case V_ASN1_UTF8STRING:
-        /* Fail for embedded NUL bytes. */
-        if (strlen((char *)str->data) != (size_t)str->length) {
-            return -1;
-        }
         ne_buffer_append(buf, (char *)str->data, str->length);
         break;
     case V_ASN1_UNIVERSALSTRING:
@@ -124,15 +103,8 @@ static int append_dirstring(ne_buffer *buf, ASN1_STRING *str)
     case V_ASN1_BMPSTRING: 
         len = ASN1_STRING_to_UTF8(&tmp, str);
         if (len > 0) {
-            /* Fail if there were embedded NUL bytes. */
-            if (strlen((char *)tmp) != (size_t)len) {
-                OPENSSL_free(tmp);
-                return -1;
-            } 
-            else {
-                ne_buffer_append(buf, (char *)tmp, len);
-                OPENSSL_free(tmp);
-            }
+            ne_buffer_append(buf, (char *)tmp, len);
+            OPENSSL_free(tmp);
             break;
         } else {
             ERR_clear_error();
@@ -147,11 +119,13 @@ static int append_dirstring(ne_buffer *buf, ASN1_STRING *str)
     return 0;
 }
 
-/* Returns a malloc-allocated version of IA5 string AS, escaped for
- * safety. */
+/* Returns a malloc-allocate version of IA5 string AS.  Really only
+ * here to prevent char * vs unsigned char * type mismatches without
+ * losing all hope at type-safety. */
 static char *dup_ia5string(const ASN1_IA5STRING *as)
 {
-    return ne_strnqdup(as->data, as->length);
+    unsigned char *data = as->data;
+    return ne_strndup((char *)data, as->length);
 }
 
 char *ne_ssl_readable_dname(const ne_ssl_dname *name)
@@ -163,16 +137,15 @@ char *ne_ssl_readable_dname(const ne_ssl_dname *name)
 
     for (n = X509_NAME_entry_count(name->dn); n > 0; n--) {
 	X509_NAME_ENTRY *ent = X509_NAME_get_entry(name->dn, n-1);
-	ASN1_OBJECT *obj = X509_NAME_ENTRY_get_object(ent);
 	
         /* Skip commonName or emailAddress except if there is no other
          * attribute in dname. */
-	if ((OBJ_cmp(obj, cname) && OBJ_cmp(obj, email)) ||
+	if ((OBJ_cmp(ent->object, cname) && OBJ_cmp(ent->object, email)) ||
             (!flag && n == 1)) {
  	    if (flag++)
 		ne_buffer_append(dump, ", ", 2);
 
-            if (append_dirstring(dump, X509_NAME_ENTRY_get_data(ent)))
+            if (append_dirstring(dump, ent->value))
                 ne_buffer_czappend(dump, "???");
 	}
     }
@@ -232,13 +205,36 @@ void ne_ssl_cert_validity_time(const ne_ssl_certificate *cert,
                                time_t *from, time_t *until)
 {
     if (from) {
-        *from = asn1time_to_timet(X509_get0_notBefore(cert->subject));
+        *from = asn1time_to_timet(X509_get_notBefore(cert->subject));
     }
     if (until) {
-        *until = asn1time_to_timet(X509_get0_notAfter(cert->subject));
+        *until = asn1time_to_timet(X509_get_notAfter(cert->subject));
     }
 }
 
+/* Return non-zero if hostname from certificate (cn) matches hostname
+ * used for session (hostname).  Doesn't implement complete RFC 2818
+ * logic; omits "f*.example.com" support for simplicity. */
+static int match_hostname(char *cn, const char *hostname)
+{
+    const char *dot;
+
+    dot = strchr(hostname, '.');
+    if (dot == NULL) {
+	char *pnt = strchr(cn, '.');
+	/* hostname is not fully-qualified; unqualify the cn. */
+	if (pnt != NULL) {
+	    *pnt = '\0';
+	}
+    }
+    else if (strncmp(cn, "*.", 2) == 0) {
+	hostname = dot + 1;
+	cn += 2;
+    }
+
+    return !ne_strcasecmp(cn, hostname);
+}
+
 /* Check certificate identity.  Returns zero if identity matches; 1 if
  * identity does not match, or <0 if the certificate had no identity.
  * If 'identity' is non-NULL, store the malloc-allocated identity in
@@ -263,7 +259,7 @@ static int check_identity(const ne_uri *server, X509 *cert, char **identity)
 	    if (nm->type == GEN_DNS) {
 		char *name = dup_ia5string(nm->d.ia5);
                 if (identity && !found) *identity = ne_strdup(name);
-		match = ne__ssl_match_hostname(name, strlen(name), hostname);
+		match = match_hostname(name, hostname);
 		ne_free(name);
 		found = 1;
             } 
@@ -347,7 +343,7 @@ static int check_identity(const ne_uri *server, X509 *cert, char **identity)
             return -1;
         }
         if (identity) *identity = ne_strdup(cname->data);
-        match = ne__ssl_match_hostname(cname->data, cname->used - 1, hostname);
+        match = match_hostname(cname->data, hostname);
         ne_buffer_destroy(cname);
     }
 
@@ -369,62 +365,6 @@ static ne_ssl_certificate *populate_cert(ne_ssl_certificate *cert, X509 *x5)
     return cert;
 }
 
-/* OpenSSL cert verification callback.  This is invoked for *each*
- * error which is encountered whilst verifying the cert chain; multiple
- * invocations for any particular cert in the chain are possible. */
-static int verify_callback(int ok, X509_STORE_CTX *ctx)
-{
-    /* OpenSSL, living in its own little happy world of global state,
-     * where userdata was just a twinkle in the eye of an API designer
-     * yet to be born.  Or... "Seriously, wtf?"  */
-    SSL *ssl = X509_STORE_CTX_get_ex_data(ctx, 
-                                          SSL_get_ex_data_X509_STORE_CTX_idx());
-    ne_session *sess = SSL_get_app_data(ssl);
-    int depth = X509_STORE_CTX_get_error_depth(ctx);
-    int err = X509_STORE_CTX_get_error(ctx);
-    int failures = 0;
-
-    /* If there's no error, nothing to do here. */
-    if (ok) return ok;
-
-    NE_DEBUG(NE_DBG_SSL, "ssl: Verify callback @ %d => %d\n", depth, err);
-
-    /* Map the error code onto any of the exported cert validation
-     * errors, if possible. */
-    switch (err) {
-    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
-    case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
-    case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
-    case X509_V_ERR_CERT_UNTRUSTED:
-    case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
-        failures |= NE_SSL_UNTRUSTED;
-        break;
-    case X509_V_ERR_CERT_NOT_YET_VALID:
-        failures |= depth > 0 ? NE_SSL_BADCHAIN : NE_SSL_NOTYETVALID;
-        break;
-    case X509_V_ERR_CERT_HAS_EXPIRED:
-        failures |= depth > 0 ? NE_SSL_BADCHAIN : NE_SSL_EXPIRED;
-        break;
-    case X509_V_OK:
-        break;
-    default:
-        /* Clear the failures bitmask so check_certificate knows this
-         * is a bailout. */
-        sess->ssl_context->failures |= NE_SSL_UNHANDLED;
-        NE_DEBUG(NE_DBG_SSL, "ssl: Unhandled verification error %d -> %s\n", 
-                 err, X509_verify_cert_error_string(err));
-        return 0;
-    }
-
-    sess->ssl_context->failures |= failures;
-
-    NE_DEBUG(NE_DBG_SSL, "ssl: Verify failures |= %d => %d\n", failures,
-             sess->ssl_context->failures);
-    
-    return 1;
-}
-
 /* Return a linked list of certificate objects from an OpenSSL chain. */
 static ne_ssl_certificate *make_chain(STACK_OF(X509) *chain)
 {
@@ -457,21 +397,17 @@ static ne_ssl_certificate *make_chain(STACK_OF(X509) *chain)
 static int check_certificate(ne_session *sess, SSL *ssl, ne_ssl_certificate *chain)
 {
     X509 *cert = chain->subject;
-    int ret, failures = sess->ssl_context->failures;
+    ASN1_TIME *notBefore = X509_get_notBefore(cert);
+    ASN1_TIME *notAfter = X509_get_notAfter(cert);
+    int ret, failures = 0;
+    long result;
     ne_uri server;
 
-    /* If the verification callback hit a case which can't be mapped
-     * to one of the exported error bits, it's treated as a hard
-     * failure rather than invoking the callback, which can't present
-     * a useful error to the user.  "Um, something is wrong.  OK?" */
-    if (failures & NE_SSL_UNHANDLED) {
-        long result = SSL_get_verify_result(ssl);
-
-        ne_set_error(sess, _("Certificate verification error: %s"),
-                    X509_verify_cert_error_string(result));
-
-        return NE_ERROR;
-    }
+    /* check expiry dates */
+    if (X509_cmp_current_time(notBefore) >= 0)
+	failures |= NE_SSL_NOTYETVALID;
+    else if (X509_cmp_current_time(notAfter) <= 0)
+	failures |= NE_SSL_EXPIRED;
 
     /* Check certificate was issued to this server; pass URI of
      * server. */
@@ -485,6 +421,38 @@ static int check_certificate(ne_session *sess, SSL *ssl, ne_ssl_certificate *cha
         return NE_ERROR;
     } else if (ret > 0) failures |= NE_SSL_IDMISMATCH;
 
+    /* get the result of the cert verification out of OpenSSL */
+    result = SSL_get_verify_result(ssl);
+
+    NE_DEBUG(NE_DBG_SSL, "Verify result: %ld = %s\n", result,
+	     X509_verify_cert_error_string(result));
+
+    switch (result) {
+    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
+    case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
+    case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
+	/* TODO: and probably more result codes here... */
+	failures |= NE_SSL_UNTRUSTED;
+	break;
+	/* ignore these, since we've already noticed them: */
+    case X509_V_ERR_CERT_NOT_YET_VALID:
+    case X509_V_ERR_CERT_HAS_EXPIRED:
+        /* cert was trusted: */
+    case X509_V_OK:
+	break;
+    default:
+	/* TODO: tricky to handle the 30-odd failure cases OpenSSL
+	 * presents here (see x509_vfy.h), and present a useful API to
+	 * the application so it in turn can then present a meaningful
+	 * UI to the user.  The only thing to do really would be to
+	 * pass back the error string, but that's not localisable.  So
+	 * just fail the verification here - better safe than
+	 * sorry. */
+	ne_set_error(sess, _("Certificate verification error: %s"),
+		     X509_verify_cert_error_string(result));
+	return NE_ERROR;
+    }
+
     if (failures == 0) {
         /* verified OK! */
         ret = NE_OK;
@@ -513,8 +481,8 @@ static ne_ssl_client_cert *dup_client_cert(const ne_ssl_client_cert *cc)
 
     populate_cert(&newcc->cert, cc->cert.subject);
 
-    X509_up_ref(cc->cert.subject);
-    EVP_PKEY_up_ref(cc->pkey);
+    cc->cert.subject->references++;
+    cc->pkey->references++;
     return newcc;
 }
 
@@ -524,7 +492,7 @@ static int provide_client_cert(SSL *ssl, X509 **cert, EVP_PKEY **pkey)
     ne_session *const sess = SSL_get_app_data(ssl);
 
     if (!sess->client_cert && sess->ssl_provide_fn) {
-	ne_ssl_dname **dnames = NULL, *dnarray = NULL;
+	ne_ssl_dname **dnames = NULL;
         int n, count = 0;
 	STACK_OF(X509_NAME) *ca_list = SSL_get_client_CA_list(ssl);
 
@@ -532,10 +500,9 @@ static int provide_client_cert(SSL *ssl, X509 **cert, EVP_PKEY **pkey)
 
         if (count > 0) {
             dnames = ne_malloc(count * sizeof(ne_ssl_dname *));
-            dnarray = ne_malloc(count * sizeof(ne_ssl_dname));
             
             for (n = 0; n < count; n++) {
-                dnames[n] = &dnarray[n];
+                dnames[n] = ne_malloc(sizeof(ne_ssl_dname));
                 dnames[n]->dn = sk_X509_NAME_value(ca_list, n);
             }
         }
@@ -544,7 +511,8 @@ static int provide_client_cert(SSL *ssl, X509 **cert, EVP_PKEY **pkey)
 	sess->ssl_provide_fn(sess->ssl_provide_ud, sess, 
                              (const ne_ssl_dname *const *)dnames, count);
         if (count) {
-            ne_free(dnarray);
+            for (n = 0; n < count; n++)
+                ne_free(dnames[n]);
             ne_free(dnames);
         }
     }
@@ -552,13 +520,12 @@ static int provide_client_cert(SSL *ssl, X509 **cert, EVP_PKEY **pkey)
     if (sess->client_cert) {
         ne_ssl_client_cert *const cc = sess->client_cert;
 	NE_DEBUG(NE_DBG_SSL, "Supplying client certificate.\n");
-	EVP_PKEY_up_ref(cc->pkey);
-	X509_up_ref(cc->cert.subject);
+	cc->pkey->references++;
+	cc->cert.subject->references++;
 	*cert = cc->cert.subject;
 	*pkey = cc->pkey;
 	return 1;
     } else {
-        sess->ssl_cc_requested = 1;
 	NE_DEBUG(NE_DBG_SSL, "No client certificate supplied.\n");
 	return 0;
     }
@@ -579,21 +546,10 @@ ne_ssl_context *ne_ssl_context_create(int mode)
         SSL_CTX_set_client_cert_cb(ctx->ctx, provide_client_cert);
         /* enable workarounds for buggy SSL server implementations */
         SSL_CTX_set_options(ctx->ctx, SSL_OP_ALL);
-        SSL_CTX_set_verify(ctx->ctx, SSL_VERIFY_PEER, verify_callback);
-#if !defined(LIBRESSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER >= 0x10101000L
-        SSL_CTX_set_post_handshake_auth(ctx->ctx, 1);
-#endif
     } else if (mode == NE_SSL_CTX_SERVER) {
         ctx->ctx = SSL_CTX_new(SSLv23_server_method());
-        SSL_CTX_set_session_cache_mode(ctx->ctx, SSL_SESS_CACHE_CLIENT);
-#ifdef SSL_OP_NO_TICKET
-        /* disable ticket support since it inhibits testing of session
-         * caching. */
-        SSL_CTX_set_options(ctx->ctx, SSL_OP_NO_TICKET);
-#endif
     } else {
-        ne_free(ctx);
-        return NULL;
+        ctx->ctx = SSL_CTX_new(SSLv2_server_method());
     }
     return ctx;
 }
@@ -617,22 +573,6 @@ void ne_ssl_context_set_flag(ne_ssl_context *ctx, int flag, int value)
     SSL_CTX_set_options(ctx->ctx, opts);
 }
 
-int ne_ssl_context_get_flag(ne_ssl_context *ctx, int flag)
-{
-    switch (flag) {
-    case NE_SSL_CTX_SSLv2:
-#ifdef OPENSSL_NO_SSL2
-        return 0;
-#else
-        return ! (SSL_CTX_get_options(ctx->ctx) & SSL_OP_NO_SSLv2);
-#endif
-    default:
-        break;
-    }
-
-    return 0;
-}
-
 int ne_ssl_context_keypair(ne_ssl_context *ctx, const char *cert,
                            const char *key)
 {
@@ -640,7 +580,7 @@ int ne_ssl_context_keypair(ne_ssl_context *ctx, const char *cert,
 
     ret = SSL_CTX_use_PrivateKey_file(ctx->ctx, key, SSL_FILETYPE_PEM);
     if (ret == 1) {
-        ret = SSL_CTX_use_certificate_chain_file(ctx->ctx, cert);
+        ret = SSL_CTX_use_certificate_file(ctx->ctx, cert, SSL_FILETYPE_PEM);
     }
 
     return ret == 1 ? 0 : -1;
@@ -673,25 +613,6 @@ void ne_ssl_context_destroy(ne_ssl_context *ctx)
     ne_free(ctx);
 }
 
-#if !defined(HAVE_SSL_SESSION_CMP) && !defined(SSL_SESSION_cmp) \
-    && defined(OPENSSL_VERSION_NUMBER) \
-    && OPENSSL_VERSION_NUMBER > 0x10000000L
-/* OpenSSL 1.0 removed SSL_SESSION_cmp for no apparent reason - hoping
- * it is reasonable to assume that comparing the session IDs is
- * sufficient. */
-static int SSL_SESSION_cmp(SSL_SESSION *a, SSL_SESSION *b)
-{
-    const unsigned char *session1_buf, *session2_buf;
-    unsigned int session1_len, session2_len;
-
-    session1_buf = SSL_SESSION_get_id(a, &session1_len);
-    session2_buf = SSL_SESSION_get_id(b, &session2_len);
-
-    return session1_len == session2_len
-        && memcmp(session1_buf, session2_buf, session1_len) == 0;
-}
-#endif
-
 /* For internal use only. */
 int ne__negotiate_ssl(ne_session *sess)
 {
@@ -701,13 +622,6 @@ int ne__negotiate_ssl(ne_session *sess)
     int freechain = 0; /* non-zero if chain should be free'd. */
 
     NE_DEBUG(NE_DBG_SSL, "Doing SSL negotiation.\n");
-    
-    /* Pass through the hostname if SNI is enabled. */
-    ctx->hostname = 
-        sess->flags[NE_SESSFLAG_TLS_SNI] ? sess->server.hostname : NULL;
-
-    sess->ssl_cc_requested = 0;
-    ctx->failures = 0;
 
     if (ne_sock_connect_ssl(sess->socket, ctx, sess)) {
 	if (ctx->sess) {
@@ -715,16 +629,9 @@ int ne__negotiate_ssl(ne_session *sess)
 	    SSL_SESSION_free(ctx->sess);
 	    ctx->sess = NULL;
 	}
-        if (sess->ssl_cc_requested) {
-            ne_set_error(sess, _("SSL handshake failed, "
-                                 "client certificate was requested: %s"),
-                         ne_sock_error(sess->socket));
-        }
-        else {
-            ne_set_error(sess, _("SSL handshake failed: %s"),
-                         ne_sock_error(sess->socket));
-        }
-        return NE_ERROR;
+	ne_set_error(sess, _("SSL negotiation failed: %s"),
+		     ne_sock_error(sess->socket));
+	return NE_ERROR;
     }	
     
     ssl = ne__sock_sslsock(sess->socket);
@@ -745,10 +652,17 @@ int ne__negotiate_ssl(ne_session *sess)
 	return NE_ERROR;
     }
 
-    if (sess->server_cert 
-        && X509_cmp(sk_X509_value(chain, 0), sess->server_cert->subject) == 0) {
-        /* Same leaf cert used as last time - no need to reverify. */
+    if (sess->server_cert) {
+        int diff = X509_cmp(sk_X509_value(chain, 0), sess->server_cert->subject);
         if (freechain) sk_X509_free(chain); /* no longer need the chain */
+	if (diff) {
+	    /* This could be a MITM attack: fail the request. */
+	    ne_set_error(sess, _("Server certificate changed: "
+				 "connection intercepted?"));
+	    return NE_ERROR;
+	} 
+	/* certificate has already passed verification: no need to
+	 * verify it again. */
     } else {
 	/* new connection: create the chain. */
         ne_ssl_certificate *cert = make_chain(chain);
@@ -852,12 +766,22 @@ static char *find_friendly_name(PKCS12 *p12)
     return name;
 }
 
-static ne_ssl_client_cert *parse_client_cert(PKCS12 *p12)
+ne_ssl_client_cert *ne_ssl_clicert_read(const char *filename)
 {
+    PKCS12 *p12;
+    FILE *fp;
     X509 *cert;
     EVP_PKEY *pkey;
     ne_ssl_client_cert *cc;
 
+    fp = fopen(filename, "rb");
+    if (fp == NULL)
+        return NULL;
+
+    p12 = d2i_PKCS12_fp(fp, NULL);
+
+    fclose(fp);
+    
     if (p12 == NULL) {
         ERR_clear_error();
         return NULL;
@@ -867,14 +791,7 @@ static ne_ssl_client_cert *parse_client_cert(PKCS12 *p12)
     if (PKCS12_parse(p12, NULL, &pkey, &cert, NULL) == 1) {
         /* Success - no password needed for decryption. */
         int len = 0;
-        unsigned char *name;
-
-        if (!cert || !pkey) {
-            PKCS12_free(p12);
-            return NULL;
-        }
-
-        name = X509_alias_get0(cert, &len);
+        unsigned char *name = X509_alias_get0(cert, &len);
         
         cc = ne_calloc(sizeof *cc);
         cc->pkey = pkey;
@@ -903,80 +820,6 @@ static ne_ssl_client_cert *parse_client_cert(PKCS12 *p12)
     }
 }
 
-ne_ssl_client_cert *ne_ssl_clicert_import(const unsigned char *buffer, 
-                                          size_t buflen)
-{
-    ne_d2i_uchar *p;
-    PKCS12 *p12;
-
-    p = buffer;
-    p12 = d2i_PKCS12(NULL, &p, buflen);
-    
-    return parse_client_cert(p12);
-}
-    
-ne_ssl_client_cert *ne_ssl_clicert_read(const char *filename)
-{
-    PKCS12 *p12;
-    FILE *fp;
-
-    fp = fopen(filename, "rb");
-    if (fp == NULL)
-        return NULL;
-
-    p12 = d2i_PKCS12_fp(fp, NULL);
-
-    fclose(fp);
-
-    return parse_client_cert(p12);
-}
-
-#ifdef HAVE_PAKCHOIS
-ne_ssl_client_cert *ne__ssl_clicert_exkey_import(const unsigned char *der,
-                                                 size_t der_len,
-                                                 const RSA_METHOD *method)
-{
-    ne_ssl_client_cert *cc;
-    ne_d2i_uchar *p;
-    X509 *x5;
-    EVP_PKEY *pubkey, *privkey;
-    RSA *rsa;
-
-    p = der;
-    x5 = d2i_X509(NULL, &p, der_len); /* p is incremented */
-    if (x5 == NULL) {
-        ERR_clear_error();
-        return NULL;
-    }
-
-    pubkey = X509_get_pubkey(x5);
-    if (EVP_PKEY_base_id(pubkey) != EVP_PKEY_RSA) {
-        X509_free(x5);
-        NE_DEBUG(NE_DBG_SSL, "ssl: Only RSA private keys are supported via PKCS#11.\n");
-        return NULL;
-    }
-
-    /* Duplicate the public parameters of the RSA key. */
-    rsa = RSAPublicKey_dup(EVP_PKEY_get0_RSA(pubkey));
-    /* Done with the copied public key. */
-    EVP_PKEY_free(pubkey);
-    
-    /* Switch to using customer RSA_METHOD for RSA object. */
-    RSA_set_method(rsa, method);
-    /* Set up new EVP_PKEY. */
-    privkey = EVP_PKEY_new();
-    EVP_PKEY_assign_RSA(privkey, rsa);
-    
-    cc = ne_calloc(sizeof *cc);
-    cc->decrypted = 1;
-    cc->pkey = privkey;
-
-    populate_cert(&cc->cert, x5);
-
-    return cc;    
-}
-#endif
-
 int ne_ssl_clicert_encrypted(const ne_ssl_client_cert *cc)
 {
     return !cc->decrypted;
@@ -1114,40 +957,10 @@ char *ne_ssl_cert_export(const ne_ssl_certificate *cert)
     return ret;
 }
 
-static const EVP_MD *hash_to_md(unsigned int flags)
-{
-    switch (flags & NE_HASH_ALGMASK) {
-    case NE_HASH_MD5: return EVP_md5();
-    case NE_HASH_SHA256: return EVP_sha256();
-#ifdef HAVE_OPENSSL11
-    case NE_HASH_SHA512: return EVP_sha512();
-    case NE_HASH_SHA512_256: return EVP_sha512_256();
-#endif
-    default: break;
-    }
-    return NULL;
-}
-
 #if SHA_DIGEST_LENGTH != 20
 # error SHA digest length is not 20 bytes
 #endif
 
-char *ne_ssl_cert_hdigest(const ne_ssl_certificate *cert, unsigned int flags)
-{
-    const EVP_MD *md = hash_to_md(flags);
-    unsigned char dig[EVP_MAX_MD_SIZE];
-    unsigned int len;
-
-    if (!md) return NULL;
-
-    if (!X509_digest(cert->subject, md, dig, &len)) {
-        ERR_clear_error();
-        return NULL;
-    }
-
-    return ne__strhash2hex(dig, len, flags);
-}
-
 int ne_ssl_cert_digest(const ne_ssl_certificate *cert, char *digest)
 {
     unsigned char sha1[EVP_MAX_MD_SIZE];
@@ -1169,35 +982,7 @@ int ne_ssl_cert_digest(const ne_ssl_certificate *cert, char *digest)
     return 0;
 }
 
-char *ne_vstrhash(unsigned int flags, va_list ap)
-{
-    EVP_MD_CTX *ctx;
-    const EVP_MD *md = hash_to_md(flags);
-    unsigned char v[EVP_MAX_MD_SIZE];
-    unsigned int vlen;
-    const char *arg;
-
-    ctx = EVP_MD_CTX_new();
-    if (!ctx) return NULL;
-
-    if (EVP_DigestInit(ctx, md) != 1) {
-        EVP_MD_CTX_free(ctx);
-        return NULL;
-    }
-
-    while ((arg = va_arg(ap, const char *)) != NULL)
-        EVP_DigestUpdate(ctx, arg, strlen(arg));
-
-    EVP_DigestFinal_ex(ctx, v, &vlen);
-    EVP_MD_CTX_free(ctx);
-
-    return ne__strhash2hex(v, vlen, flags);
-}
-
-#if defined(NE_HAVE_TS_SSL) && OPENSSL_VERSION_NUMBER < 0x10100000L
-/* From OpenSSL 1.1.0 locking callbacks are no longer needed. */
-#define WITH_OPENSSL_LOCKING (1)
-
+#ifdef NE_HAVE_TS_SSL
 /* Implementation of locking callbacks to make OpenSSL thread-safe.
  * If the OpenSSL API was better designed, this wouldn't be necessary.
  * In OpenSSL releases without CRYPTO_set_idptr_callback, it's not
@@ -1205,25 +990,17 @@ char *ne_vstrhash(unsigned int flags, va_list ap)
  * it's necessary to cast from a pthread_t to an unsigned long at some
  * point.  */
 
-#ifndef _WIN32
 static pthread_mutex_t *locks;
-#else
-static HANDLE *locks;
-#endif
 static size_t num_locks;
 
 #ifndef HAVE_CRYPTO_SET_IDPTR_CALLBACK
 /* Named to be obvious when it shows up in a backtrace. */
 static unsigned long thread_id_neon(void)
 {
-#ifndef _WIN32
     /* This will break if pthread_t is a structure; upgrading OpenSSL
      * >= 0.9.9 (which does not require this callback) is the only
      * solution.  */
     return (unsigned long) pthread_self();
-#else
-    return (unsigned long) GetCurrentThreadId();
-#endif
 }
 #endif
 
@@ -1232,25 +1009,19 @@ static unsigned long thread_id_neon(void)
 static void thread_lock_neon(int mode, int n, const char *file, int line)
 {
     if (mode & CRYPTO_LOCK) {
-#ifndef _WIN32
         if (pthread_mutex_lock(&locks[n])) {
-#else
-        if (WaitForSingleObject(locks[n], INFINITE)) {
-#endif
             abort();
         }
     }
     else {
-#ifndef _WIN32
         if (pthread_mutex_unlock(&locks[n])) {
-#else
-        if (!ReleaseMutex(locks[n])) {
-#endif
             abort();
         }
     }
 }
 
+#endif
+
 /* ID_CALLBACK_IS_{NEON,OTHER} evaluate as true if the currently
  * registered OpenSSL ID callback is the neon function (_NEON), or has
  * been overwritten by some other app (_OTHER). */
@@ -1261,18 +1032,15 @@ static void thread_lock_neon(int mode, int n, const char *file, int line)
 #define ID_CALLBACK_IS_OTHER (CRYPTO_get_id_callback() != NULL)
 #define ID_CALLBACK_IS_NEON (CRYPTO_get_id_callback() == thread_id_neon)
 #endif
-        
-#endif /* NE_HAVE_TS_SSL && OPENSSL_VERSION_NUMBER < 1.1.1 */
 
 int ne__ssl_init(void)
 {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
     CRYPTO_malloc_init();
     SSL_load_error_strings();
     SSL_library_init();
     OpenSSL_add_all_algorithms();
 
-#ifdef WITH_OPENSSL_LOCKING
+#ifdef NE_HAVE_TS_SSL
     /* If some other library has already come along and set up the
      * thread-safety callbacks, then it must be presumed that the
      * other library will have a longer lifetime in the process than
@@ -1295,11 +1063,7 @@ int ne__ssl_init(void)
 
         locks = malloc(num_locks * sizeof *locks);
         for (n = 0; n < num_locks; n++) {
-#ifndef _WIN32
             if (pthread_mutex_init(&locks[n], NULL)) {
-#else
-            if ((locks[n] = CreateMutex(NULL, FALSE, NULL)) == NULL) {
-#endif
                 NE_DEBUG(NE_DBG_SOCKET, "ssl: Failed to initialize pthread mutex.\n");
                 return -1;
             }
@@ -1309,7 +1073,6 @@ int ne__ssl_init(void)
                  "for %" NE_FMT_SIZE_T " locks.\n", num_locks);
     }
 #endif
-#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
     return 0;
 }
@@ -1319,7 +1082,7 @@ void ne__ssl_exit(void)
     /* Cannot call ERR_free_strings() etc here in case any other code
      * in the process using OpenSSL. */
 
-#ifdef WITH_OPENSSL_LOCKING
+#ifdef NE_HAVE_TS_SSL
     /* Only unregister the callbacks if some *other* library has not
      * come along in the mean-time and trampled over the callbacks
      * installed by neon. */
@@ -1333,11 +1096,7 @@ void ne__ssl_exit(void)
         CRYPTO_set_locking_callback(NULL);
 
         for (n = 0; n < num_locks; n++) {
-#ifndef _WIN32
             pthread_mutex_destroy(&locks[n]);
-#else
-            CloseHandle(locks[n]);
-#endif
         }
 
         free(locks);
