@@ -1,6 +1,6 @@
 /* 
    HTTP Request Handling
-   Copyright (C) 1999-2009, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -71,9 +71,6 @@ struct ne_session_s {
     const ne_inet_addr **addrlist;
     size_t numaddrs, curaddr;
 
-    /* Local address to which sockets should be bound. */
-    const ne_inet_addr *local_addr;
-
     /* Settings */
     int use_proxy; /* do we have a proxy server? */
     int use_ssl; /* whether a secure connection is required */
@@ -90,8 +87,7 @@ struct ne_session_s {
     int rdtimeout, cotimeout; /* read, connect timeouts. */
 
     struct hook *create_req_hooks, *pre_send_hooks, *post_send_hooks,
-        *post_headers_hooks, *destroy_req_hooks, *destroy_sess_hooks, 
-        *close_conn_hooks, *private;
+        *post_headers_hooks, *destroy_req_hooks, *destroy_sess_hooks, *private;
 
     char *user_agent; /* full User-Agent: header field */
 
@@ -99,9 +95,6 @@ struct ne_session_s {
     ne_ssl_client_cert *client_cert;
     ne_ssl_certificate *server_cert;
     ne_ssl_context *ssl_context;
-    int ssl_cc_requested; /* set to non-zero if a client cert was
-                           * requested during initial handshake, but
-                           * none could be provided. */
 #endif
 
     /* Server cert verification callback: */
@@ -127,18 +120,4 @@ int ne__negotiate_ssl(ne_session *sess);
 /* Set the session error appropriate for SSL verification failures. */
 void ne__ssl_set_verify_err(ne_session *sess, int failures);
 
-/* Return non-zero if hostname from certificate (cn) matches hostname
- * used for session (hostname); follows RFC2818 logic. */
-int ne__ssl_match_hostname(const char *cn, size_t cnlen, const char *hostname);
-
-/* Return a malloc-allocated copy of 'data', of length 'len', with all
- * non-ASCII bytes, and ASCII control characters escaped.  (Note that
- * the escaping includes the NUL byte). */
-char *ne__strnqdup(const unsigned char *data, size_t len);
-
-/* Append 'len' bytes of 'data' to buf.  All non-ASCII bytes, and
- * ASCII control characters, are escaped.  (Note that this includes
- * the NUL byte). */
-void ne__buffer_qappend(ne_buffer *buf, const unsigned char *data, size_t len);
-
 #endif /* HTTP_PRIVATE_H */
