@@ -1,6 +1,6 @@
 /* 
    HTTP request/response handling
-   Copyright (C) 1999-2021, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2007, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -72,8 +72,6 @@ struct field {
     struct field *next;
 };
 
-/* Maximum number of interim responses. */
-#define MAX_INTERIM_RESPONSES (128)
 /* Maximum number of header fields per response: */
 #define MAX_HEADER_FIELDS (100)
 /* Size of hash table; 43 is the smallest prime for which the common
@@ -84,12 +82,11 @@ struct field {
 
 /* pre-calculated hash values for given header names: */
 #define HH_HV_CONNECTION        (0x14)
-#define HH_HV_PROXY_CONNECTION  (0x1A)
 #define HH_HV_CONTENT_LENGTH    (0x13)
 #define HH_HV_TRANSFER_ENCODING (0x07)
 
 struct ne_request_s {
-    char *method, *target; /* method and request-target */
+    char *method, *uri; /* method and Request-URI */
 
     ne_buffer *headers; /* request headers */
 
@@ -192,7 +189,7 @@ static int aborted(ne_request *req, const char *doing, ssize_t code)
 
     switch(code) {
     case NE_SOCK_CLOSED:
-	if (sess->nexthop->proxy != PROXY_NONE) {
+	if (sess->use_proxy) {
 	    ne_set_error(sess, _("%s: connection was closed by proxy server"),
 			 doing);
 	} else {
@@ -286,36 +283,11 @@ static ssize_t body_fd_send(void *userdata, char *buffer, size_t count)
     ne_request *req = userdata;
 
     if (count) {
-        ssize_t ret;
-
         if (req->body.file.remain == 0)
             return 0;
-
-        /* Casts here are necessary for LFS platforms for safe and
-         * warning-free assignment/comparison between 32-bit size_t
-         * and 64-bit off64_t: */
-        if ((ne_off_t)count > req->body.file.remain)
-            count = (size_t)req->body.file.remain;
-        
-        ret = read(req->body.file.fd, buffer, count);
-        if (ret > 0) {
-            req->body.file.remain -= ret;
-            return ret;
-        }
-        else if (ret == 0) {
-            ne_set_error(req->session, 
-                         _("Premature EOF in request body file"));
-        }
-        else if (ret < 0) {
-            char err[200];
-            int errnum = errno;
-
-            ne_set_error(req->session, 
-                         _("Failed reading request body file: %s"),
-                         ne_strerror(errnum, err, sizeof err));
-        }
-
-        return -1;
+        if ((off_t)count > req->body.file.remain)
+            count = req->body.file.remain;
+	return read(req->body.file.fd, buffer, count);
     } else {
         ne_off_t newoff;
 
@@ -331,7 +303,7 @@ static ssize_t body_fd_send(void *userdata, char *buffer, size_t count)
                 /* errno was set */
                 ne_strerror(errno, err, sizeof err);
             } else {
-                ne_strnzcpy(err, _("offset invalid"), sizeof err);
+                strcpy(err, _("offset invalid"));
             }
             ne_snprintf(offstr, sizeof offstr, "%" FMT_NE_OFF_T,
                         req->body.file.offset);
@@ -359,14 +331,6 @@ static ssize_t body_fd_send(void *userdata, char *buffer, size_t count)
 ((((code) == NE_SOCK_CLOSED || (code) == NE_SOCK_RESET || \
  (code) == NE_SOCK_TRUNC) && retry) ? NE_RETRY : (acode))
 
-/* For sending chunks, an 8-byte prefix is reserved at the beginning
- * of the buffer.  This is large enough for a trailing \r\n for the
- * previous chunk, the chunk size, and the \r\n following the
- * chunk-size. */
-#define CHUNK_OFFSET (8)
-#define CHUNK_TERM "\r\n0\r\n\r\n"
-#define CHUNK_NULL_TERM "0\r\n\r\n"
-
 /* Sends the request body; returns 0 on success or an NE_* error code.
  * If retry is non-zero; will return NE_RETRY on persistent connection
  * timeout.  On error, the session error string is set and the
@@ -374,28 +338,13 @@ static ssize_t body_fd_send(void *userdata, char *buffer, size_t count)
 static int send_request_body(ne_request *req, int retry)
 {
     ne_session *const sess = req->session;
-    char buffer[NE_BUFSIZ], *start;
+    char buffer[NE_BUFSIZ];
     ssize_t bytes;
-    size_t buflen;
-    int chunked = req->body_length < 0, chunknum = 0;
-    int ret;
 
     NE_DEBUG(NE_DBG_HTTP, "Sending request body:\n");
 
-    /* Set up status union and (start, buflen) as the buffer to be
-     * passed the supplied callback. */
-    if (chunked) {
-        start = buffer + CHUNK_OFFSET;
-        buflen = sizeof(buffer) - CHUNK_OFFSET;
-        req->session->status.sr.total = -1;
-    }
-    else {
-        start = buffer;
-        buflen = sizeof buffer;
-        req->session->status.sr.total = req->body_length;
-    }
-
     req->session->status.sr.progress = 0;
+    req->session->status.sr.total = req->body_length;
     notify_status(sess, ne_status_sending);
     
     /* tell the source to start again from the beginning. */
@@ -404,23 +353,8 @@ static int send_request_body(ne_request *req, int retry)
         return NE_ERROR;
     }
     
-    while ((bytes = req->body_cb(req->body_ud, start, buflen)) > 0) {
-        req->session->status.sr.progress += bytes;
-        if (chunked) {
-            /* Overwrite the buffer prefix with the appropriate chunk
-             * size; since ne_snprintf always NUL-terminates, the \n
-             * is omitted and placed over the NUL afterwards. */
-            if (chunknum++ == 0)
-                ne_snprintf(buffer, CHUNK_OFFSET, 
-                            "%06x\r", (unsigned)bytes);
-            else
-                ne_snprintf(buffer, CHUNK_OFFSET, 
-                            "\r\n%04x\r", (unsigned)bytes);
-            buffer[CHUNK_OFFSET - 1] = '\n';
-            bytes += CHUNK_OFFSET;
-        }
-        ret = ne_sock_fullwrite(sess->socket, buffer, bytes);
-
+    while ((bytes = req->body_cb(req->body_ud, buffer, sizeof buffer)) > 0) {
+	int ret = ne_sock_fullwrite(sess->socket, buffer, bytes);
         if (ret < 0) {
             int aret = aborted(req, _("Could not send request body"), ret);
             return RETRY_RET(retry, ret, aret);
@@ -431,69 +365,46 @@ static int send_request_body(ne_request *req, int retry)
 		 bytes, (int)bytes, buffer);
 
         /* invoke progress callback */
+        req->session->status.sr.progress += bytes;
         notify_status(sess, ne_status_sending);
     }
 
-    if (bytes) {
+    if (bytes == 0) {
+        return NE_OK;
+    } else {
         NE_DEBUG(NE_DBG_HTTP, "Request body provider failed with "
                  "%" NE_FMT_SSIZE_T "\n", bytes);
         ne_close_connection(sess);
         return NE_ERROR;
     }
-
-    if (chunked) {
-        if (chunknum == 0)
-            ret = ne_sock_fullwrite(sess->socket, CHUNK_NULL_TERM, 
-                                    sizeof(CHUNK_NULL_TERM) - 1);
-        else
-            ret = ne_sock_fullwrite(sess->socket, CHUNK_TERM, 
-                                    sizeof(CHUNK_TERM) - 1);
-        if (ret < 0) {
-            int aret = aborted(req, _("Could not send chunked "
-                                      "request terminator"), ret);
-            return RETRY_RET(retry, ret, aret);
-        }
-    }
-    
-    return NE_OK;
 }
 
-/* Set up buffer for initial request headers. */
-static ne_buffer *initial_request_headers(ne_request *req) 
+/* Lob the User-Agent, connection and host headers in to the request
+ * headers */
+static void add_fixed_headers(ne_request *req) 
 {
-    ne_session *const sess = req->session;
-    ne_buffer *hdrs = ne_buffer_create();
-
-    if (sess->user_agent) {
-        ne_buffer_zappend(hdrs, sess->user_agent);
+    if (req->session->user_agent) {
+        ne_buffer_zappend(req->headers, req->session->user_agent);
     }
 
     /* If persistent connections are disabled, just send Connection:
      * close; otherwise, send Connection: Keep-Alive to pre-1.1 origin
      * servers to try harder to get a persistent connection, except if
      * using a proxy as per 2068§19.7.1.  Always add TE: trailers. */
-    if (!sess->flags[NE_SESSFLAG_PERSIST]) {
-       ne_buffer_czappend(hdrs, "Connection: TE, close" EOL);
-    } 
-    else if (!sess->is_http11 && !sess->any_proxy_http) {
-        ne_buffer_czappend(hdrs, 
-                           "Keep-Alive: " EOL
-                          "Connection: TE, Keep-Alive" EOL);
-    } 
-    else if (!req->session->is_http11 && !sess->any_proxy_http) {
-        ne_buffer_czappend(hdrs, 
-                           "Keep-Alive: " EOL
-                           "Proxy-Connection: Keep-Alive" EOL
-                           "Connection: TE" EOL);
-    } 
-    else {
-        ne_buffer_czappend(hdrs, "Connection: TE" EOL);
+    if (!req->session->flags[NE_SESSFLAG_PERSIST]) {
+       ne_buffer_czappend(req->headers,
+                          "Connection: TE, close" EOL
+                          "TE: trailers" EOL);
+    } else if (!req->session->is_http11 && !req->session->use_proxy) {
+        ne_buffer_czappend(req->headers, 
+                          "Keep-Alive: " EOL
+                          "Connection: TE, Keep-Alive" EOL
+                          "TE: trailers" EOL);
+    } else {
+        ne_buffer_czappend(req->headers, 
+                           "Connection: TE" EOL
+                           "TE: trailers" EOL);
     }
-
-    ne_buffer_concat(hdrs, "TE: trailers" EOL "Host: ", 
-                     req->session->server.hostport, EOL, NULL);
-
-    return hdrs;
 }
 
 int ne_accept_always(void *userdata, ne_request *req, const ne_status *st)
@@ -512,34 +423,32 @@ ne_request *ne_request_create(ne_session *sess,
     ne_request *req = ne_calloc(sizeof *req);
 
     req->session = sess;
+    req->headers = ne_buffer_create();
     
     /* Presume the method is idempotent by default. */
     req->flags[NE_REQFLAG_IDEMPOTENT] = 1;
-    /* Expect-100 default follows the corresponding session flag. */
-    req->flags[NE_REQFLAG_EXPECT100] = sess->flags[NE_SESSFLAG_EXPECT100];
 
     /* Add in the fixed headers */
-    req->headers = initial_request_headers(req);
+    add_fixed_headers(req);
 
     /* Set the standard stuff */
     req->method = ne_strdup(method);
     req->method_is_head = (strcmp(method, "HEAD") == 0);
 
-    /* Only use an absoluteURI here when we might be using an HTTP
-     * proxy, and SSL is in use: some servers can't parse them. */
-    if (sess->any_proxy_http && !req->session->use_ssl && path[0] == '/')
-        req->target = ne_concat(req->session->scheme, "://",
-                                req->session->server.hostport,
-                                path, NULL);
+    /* Only use an absoluteURI here when absolutely necessary: some
+     * servers can't parse them. */
+    if (req->session->use_proxy && !req->session->use_ssl && path[0] == '/')
+	req->uri = ne_concat(req->session->scheme, "://", 
+			     req->session->server.hostport, path, NULL);
     else
-        req->target = ne_strdup(path);
+	req->uri = ne_strdup(path);
 
     {
 	struct hook *hk;
 
 	for (hk = sess->create_req_hooks; hk != NULL; hk = hk->next) {
 	    ne_create_request_fn fn = (ne_create_request_fn)hk->fn;
-	    fn(req, hk->userdata, req->method, req->target);
+	    fn(req, hk->userdata, method, req->uri);
 	}
     }
 
@@ -550,12 +459,7 @@ ne_request *ne_request_create(ne_session *sess,
 static void set_body_length(ne_request *req, ne_off_t length)
 {
     req->body_length = length;
-
-    if (length >= 0)
-        ne_print_request_header(req, "Content-Length", "%" FMT_NE_OFF_T, length);
-    else /* length < 0 => chunked body */
-        ne_add_request_header(req, "Transfer-Encoding", "chunked");
-
+    ne_print_request_header(req, "Content-Length", "%" FMT_NE_OFF_T, length);
 }
 
 void ne_set_request_body_buffer(ne_request *req, const char *buffer,
@@ -589,14 +493,14 @@ void ne_set_request_body_fd(ne_request *req, int fd,
 
 void ne_set_request_flag(ne_request *req, ne_request_flag flag, int value)
 {
-    if (flag < (ne_request_flag)NE_REQFLAG_LAST) {
+    if (flag < NE_SESSFLAG_LAST) {
         req->flags[flag] = value;
     }
 }
 
 int ne_get_request_flag(ne_request *req, ne_request_flag flag)
 {
-    if (flag < (ne_request_flag)NE_REQFLAG_LAST) {
+    if (flag < NE_REQFLAG_LAST) {
         return req->flags[flag];
     }
     return -1;
@@ -727,7 +631,7 @@ void ne_request_destroy(ne_request *req)
     struct body_reader *rdr, *next_rdr;
     struct hook *hk, *next_hk;
 
-    ne_free(req->target);
+    ne_free(req->uri);
     ne_free(req->method);
 
     for (rdr = req->body_readers; rdr != NULL; rdr = next_rdr) {
@@ -762,7 +666,7 @@ void ne_request_destroy(ne_request *req)
 /* Reads a block of the response into BUFFER, which is of size
  * *BUFLEN.  Returns zero on success or non-zero on error.  On
  * success, *BUFLEN is updated to be the number of bytes read into
- * BUFFER (which will be 0 to indicate the end of the response).  On
+ * BUFFER (which will be 0 to indicate the end of the repsonse).  On
  * error, the connection is closed and the session error string is
  * set.  */
 static int read_response_block(ne_request *req, struct ne_response *resp, 
@@ -887,16 +791,16 @@ static ne_buffer *build_request(ne_request *req)
     struct hook *hk;
     ne_buffer *buf = ne_buffer_create();
 
-    /* Add Request-Line and headers: */
-    ne_buffer_concat(buf, req->method, " ", req->target, " HTTP/1.1" EOL,
-                     NULL);
-
+    /* Add Request-Line and Host header: */
+    ne_buffer_concat(buf, req->method, " ", req->uri, " HTTP/1.1" EOL,
+		     "Host: ", req->session->server.hostport, EOL, NULL);
+    
     /* Add custom headers: */
     ne_buffer_append(buf, req->headers->data, ne_buffer_size(req->headers));
 
-    if (req->body_length && req->flags[NE_REQFLAG_EXPECT100]) {
-        ne_buffer_czappend(buf, "Expect: 100-continue\r\n");
-    }
+#define E100 "Expect: 100-continue" EOL
+    if (req->flags[NE_REQFLAG_EXPECT100])
+	ne_buffer_append(buf, E100, strlen(E100));
 
     NE_DEBUG(NE_DBG_HTTP, "Running pre_send hooks\n");
     for (hk = req->session->pre_send_hooks; hk!=NULL; hk = hk->next) {
@@ -904,7 +808,7 @@ static ne_buffer *build_request(ne_request *req)
 	fn(req, hk->userdata, buf);
     }
     
-    ne_buffer_czappend(buf, "\r\n");
+    ne_buffer_append(buf, "\r\n", 2);
     return buf;
 }
 
@@ -917,7 +821,7 @@ static void dump_request(const char *request)
 	/* Display everything mode */
 	NE_DEBUG(NE_DBG_HTTP, "Sending request headers:\n%s", request);
     } else if (ne_debug_mask & NE_DBG_HTTP) {
-	/* Blank out the Authorization parameters */
+	/* Blank out the Authorization paramaters */
 	char *reqdebug = ne_strdup(request), *pnt = reqdebug;
 	while ((pnt = strstr(pnt, "Authorization: ")) != NULL) {
 	    for (pnt += 15; *pnt != '\r' && *pnt != '\0'; pnt++) {
@@ -945,12 +849,6 @@ static inline void strip_eol(char *buf, ssize_t *len)
     }
 }
 
-#ifdef NE_HAVE_SSL
-#define SSL_CC_REQUESTED(_r) (_r->session->ssl_cc_requested)
-#else
-#define SSL_CC_REQUESTED(_r) (0)
-#endif
-
 /* Read and parse response status-line into 'status'.  'retry' is non-zero
  * if an NE_RETRY should be returned if an EOF is received. */
 static int read_status_line(ne_request *req, ne_status *status, int retry)
@@ -960,11 +858,8 @@ static int read_status_line(ne_request *req, ne_status *status, int retry)
 
     ret = ne_sock_readline(req->session->socket, buffer, sizeof req->respbuf);
     if (ret <= 0) {
-        const char *errstr = SSL_CC_REQUESTED(req)
-            ? _("Could not read status line (TLS client certificate was requested)")
-            : _("Could not read status line");
-        int aret = aborted(req, errstr, ret);
-        return RETRY_RET(retry, ret, aret);
+	int aret = aborted(req, _("Could not read status line"), ret);
+	return RETRY_RET(retry, ret, aret);
     }
     
     NE_DEBUG(NE_DBG_HTTP, "[status-line] < %s", buffer);
@@ -1017,7 +912,6 @@ static int send_request(ne_request *req, const ne_buffer *request)
     ne_status *const status = &req->status;
     int sentbody = 0; /* zero until body has been sent. */
     int ret, retry; /* retry non-zero whilst the request should be retried */
-    unsigned count;
     ssize_t sret;
 
     /* Send the Request-Line and headers */
@@ -1036,7 +930,7 @@ static int send_request(ne_request *req, const ne_buffer *request)
 	return RETRY_RET(retry, sret, aret);
     }
     
-    if (!req->flags[NE_REQFLAG_EXPECT100] && req->body_length) {
+    if (!req->flags[NE_REQFLAG_EXPECT100] && req->body_length > 0) {
 	/* Send request body, if not using 100-continue. */
 	ret = send_request_body(req, retry);
 	if (ret) {
@@ -1046,29 +940,23 @@ static int send_request(ne_request *req, const ne_buffer *request)
     
     NE_DEBUG(NE_DBG_HTTP, "Request sent; retry is %d.\n", retry);
 
-    /* Loop eating interim 1xx responses; RFC 7231§6.2 says clients
-     * MUST be able to parse unsolicited interim responses. */
-    for (count = 0; count < MAX_INTERIM_RESPONSES
-             && (ret = read_status_line(req, status, retry)) == NE_OK
-             && status->klass == 1; count++) {
-	NE_DEBUG(NE_DBG_HTTP, "[req] Interim %d response %d.\n",
-                 status->code, count);
+    /* Loop eating interim 1xx responses (RFC2616 says these MAY be
+     * sent by the server, even if 100-continue is not used). */
+    while ((ret = read_status_line(req, status, retry)) == NE_OK 
+	   && status->klass == 1) {
+	NE_DEBUG(NE_DBG_HTTP, "Interim %d response.\n", status->code);
 	retry = 0; /* successful read() => never retry now. */
 	/* Discard headers with the interim response. */
 	if ((ret = discard_headers(req)) != NE_OK) break;
 
 	if (req->flags[NE_REQFLAG_EXPECT100] && (status->code == 100)
-            && req->body_length && !sentbody) {
+            && req->body_length > 0 && !sentbody) {
 	    /* Send the body after receiving the first 100 Continue */
 	    if ((ret = send_request_body(req, 0)) != NE_OK) break;	    
 	    sentbody = 1;
 	}
     }
 
-    if (count == MAX_INTERIM_RESPONSES) {
-        return aborted(req, _("Too many interim responses"), 0);
-    }
-
     return ret;
 }
 
@@ -1123,8 +1011,8 @@ static int read_message_header(ne_request *req, char *buf, size_t buflen)
 	
 	/* assert(buf[0] == ch), which implies len(buf) > 0.
 	 * Otherwise the TCP stack is lying, but we'll be paranoid.
-	 * This might be a \t, so replace it with a space for ease of
-	 * parsing; this is permitted by RFC 7230§3.5. */
+	 * This might be a \t, so replace it with a space to be
+	 * friendly to applications (2616 says we MAY do this). */
 	if (n) buf[0] = ' ';
 
 	/* ready for the next header. */
@@ -1220,9 +1108,11 @@ static int read_response_headers(ne_request *req)
 }
 
 /* Perform any necessary DNS lookup for the host given by *info;
- * returns NE_ code with error string set on error. */
+ * return NE_ code. */
 static int lookup_host(ne_session *sess, struct host_info *info)
 {
+    if (sess->addrlist) return NE_OK;
+
     NE_DEBUG(NE_DBG_HTTP, "Doing DNS lookup on %s...\n", info->hostname);
     sess->status.lu.hostname = info->hostname;
     notify_status(sess, ne_status_lookup);
@@ -1247,7 +1137,7 @@ int ne_begin_request(ne_request *req)
     const ne_status *const st = &req->status;
     const char *value;
     struct hook *hk;
-    int ret, forced_closure = 0;
+    int ret;
 
     /* If a non-idempotent request is sent on a persisted connection,
      * then it is impossible to distinguish between a server failure
@@ -1300,7 +1190,6 @@ int ne_begin_request(ne_request *req)
 
             if (strcmp(token, "close") == 0) {
                 req->can_persist = 0;
-                forced_closure = 1;
             } else if (strcmp(token, "keep-alive") == 0) {
                 req->can_persist = 1;
             } else if (!req->session->is_http11
@@ -1314,42 +1203,24 @@ int ne_begin_request(ne_request *req)
         ne_free(vcopy);
     }
 
-    /* Support "Proxy-Connection: keep-alive" for compatibility with
-     * some HTTP/1.0 proxies; it is risky to do this, because an
-     * intermediary proxy may not support this HTTP/1.0 extension, but
-     * will not strip the header either.  Persistent connection
-     * support is enabled based on the presence of this header if:
-     * a) it is *necessary* to do so due to the use of a connection-auth
-     * scheme, and
-     * b) connection closure was not forced via "Connection: close".  */
-    if (req->session->nexthop->proxy == PROXY_HTTP && !req->session->is_http11
-        && !forced_closure && req->session->flags[NE_SESSFLAG_CONNAUTH]) {
-        value = get_response_header_hv(req, HH_HV_PROXY_CONNECTION,
-                                       "proxy-connection");
-        if (value && ne_strcasecmp(value, "keep-alive") == 0) {
-            NE_DEBUG(NE_DBG_HTTP, "req: Using persistent connection "
-                     "for HTTP/1.0 proxy requiring conn-auth hack.\n");
-            req->can_persist = 1;
-        }
-    }
-
     /* Decide which method determines the response message-length per
-     * RFC 7230§3.3.3, method cases follow: */
+     * 2616§4.4 (multipart/byteranges is not supported): */
 
 #ifdef NE_HAVE_SSL
-    /* Case (2) is special-cased first for CONNECT: the response has
-     * no body, and the connection can persist. */
+    /* Special case for CONNECT handling: the response has no body,
+     * and the connection can persist. */
     if (req->session->in_connect && st->klass == 2) {
 	req->resp.mode = R_NO_BODY;
 	req->can_persist = 1;
     } else
 #endif
-    /* Case (1), HEAD requests and 204, 304 responses have no response
-     * body, regardless of what headers are present. */
+    /* HEAD requests and 204, 304 responses have no response body,
+     * regardless of what headers are present. */
     if (req->method_is_head || st->code == 204 || st->code == 304) {
     	req->resp.mode = R_NO_BODY;
     }
-    /* Case (3), chunked transer-encoding.. */
+    /* Broken intermediaries exist which use "transfer-encoding: identity"
+     * to mean "no transfer-coding".  So that case must be ignored. */
     else if ((value = get_response_header_hv(req, HH_HV_TRANSFER_ENCODING,
                                              "transfer-encoding")) != NULL
              && ne_strcasecmp(value, "identity") != 0) {
@@ -1361,24 +1232,18 @@ int ne_begin_request(ne_request *req)
         else {
             return aborted(req, _("Unknown transfer-coding in response"), 0);
         }
-    }
-    /* Case (4) and (5), content-length delimited. */
+    } 
     else if ((value = get_response_header_hv(req, HH_HV_CONTENT_LENGTH,
                                              "content-length")) != NULL) {
-        char *endptr = NULL;
-        ne_off_t len = ne_strtoff(value, &endptr, 10);
-
-        if (*value && len != NE_OFFT_MAX && len >= 0 && endptr && *endptr == '\0') {
+        ne_off_t len = ne_strtoff(value, NULL, 10);
+        if (len != NE_OFFT_MAX && len >= 0) {
             req->resp.mode = R_CLENGTH;
             req->resp.body.clen.total = req->resp.body.clen.remain = len;
-        }
-        else {
-            /* Per case (4), an invalid C-L must be treated as an error. */
+        } else {
+            /* fail for an invalid content-length header. */
             return aborted(req, _("Invalid Content-Length in response"), 0);
         }
-    }
-    /* Case (7), response delimited by EOF. */
-    else {
+    } else {
         req->resp.mode = R_TILLEOF; /* otherwise: read-till-eof mode */
     }
     
@@ -1534,32 +1399,40 @@ static int proxy_tunnel(ne_session *sess)
 #endif
 
 /* Return the first resolved address for the given host. */
-static const ne_inet_addr *resolve_first(struct host_info *host)
+static const ne_inet_addr *resolve_first(ne_session *sess, 
+                                         struct host_info *host)
 {
-    return host->network ? host->network : ne_addr_first(host->address);
+    if (sess->addrlist) {
+        sess->curaddr = 0;
+        return sess->addrlist[0];
+    } else {
+        return ne_addr_first(host->address);
+    }
 }
 
 /* Return the next resolved address for the given host or NULL if
  * there are no more addresses. */
-static const ne_inet_addr *resolve_next(struct host_info *host)
+static const ne_inet_addr *resolve_next(ne_session *sess,
+                                        struct host_info *host)
 {
-    return host->network ? NULL : ne_addr_next(host->address);
+    if (sess->addrlist) {
+        if (sess->curaddr++ < sess->numaddrs)
+            return sess->addrlist[sess->curaddr];
+        else
+            return NULL;
+    } else {
+        return ne_addr_next(host->address);
+    }
 }
 
 /* Make new TCP connection to server at 'host' of type 'name'.  Note
  * that once a connection to a particular network address has
  * succeeded, that address will be used first for the next attempt to
  * connect. */
-static int do_connect(ne_session *sess, struct host_info *host)
+static int do_connect(ne_session *sess, struct host_info *host, const char *err)
 {
     int ret;
 
-    /* Resolve hostname if necessary. */
-    if (host->address == NULL && host->network == NULL) {
-        ret = lookup_host(sess, host);
-        if (ret) return ret;
-    }
-
     if ((sess->socket = ne_sock_create()) == NULL) {
         ne_set_error(sess, _("Could not create socket"));
         return NE_ERROR;
@@ -1568,11 +1441,8 @@ static int do_connect(ne_session *sess, struct host_info *host)
     if (sess->cotimeout)
 	ne_sock_connect_timeout(sess->socket, sess->cotimeout);
 
-    if (sess->local_addr)
-        ne_sock_prebind(sess->socket, sess->local_addr, 0);
-
     if (host->current == NULL)
-	host->current = resolve_first(host);
+	host->current = resolve_first(sess, host);
 
     sess->status.ci.hostname = host->hostname;
 
@@ -1582,148 +1452,56 @@ static int do_connect(ne_session *sess, struct host_info *host)
 #ifdef NE_DEBUGGING
 	if (ne_debug_mask & NE_DBG_HTTP) {
 	    char buf[150];
-	    NE_DEBUG(NE_DBG_HTTP, "req: Connecting to %s:%u\n",
-		     ne_iaddr_print(host->current, buf, sizeof buf),
-                     host->port);
+	    NE_DEBUG(NE_DBG_HTTP, "Connecting to %s\n",
+		     ne_iaddr_print(host->current, buf, sizeof buf));
 	}
 #endif
 	ret = ne_sock_connect(sess->socket, host->current, host->port);
     } while (ret && /* try the next address... */
-	     (host->current = resolve_next(host)) != NULL);
+	     (host->current = resolve_next(sess, host)) != NULL);
 
     if (ret) {
-        const char *msg;
-
-        if (host->proxy == PROXY_NONE)
-            msg = _("Could not connect to server");
-        else
-            msg = _("Could not connect to proxy server");
-
-        ne_set_error(sess, "%s: %s", msg, ne_sock_error(sess->socket));
+        ne_set_error(sess, "%s: %s", err, ne_sock_error(sess->socket));
         ne_sock_close(sess->socket);
-	return ret == NE_SOCK_TIMEOUT ? NE_TIMEOUT : NE_CONNECT;
+	return NE_CONNECT;
     }
 
+    notify_status(sess, ne_status_connected);
+    
     if (sess->rdtimeout)
 	ne_sock_read_timeout(sess->socket, sess->rdtimeout);
 
-    notify_status(sess, ne_status_connected);
-    sess->nexthop = host;
-
     sess->connected = 1;
     /* clear persistent connection flag. */
     sess->persisted = 0;
     return NE_OK;
 }
 
-/* For a SOCKSv4 proxy only, the IP address of the origin server (in
- * addition to the proxy) must be known, and must be an IPv4 address.
- * Returns NE_*; connection closed and error string set on error. */
-static int socks_origin_lookup(ne_session *sess)
-{
-    const ne_inet_addr *ia;
-    int ret;
-
-    ret = lookup_host(sess, &sess->server);
-    if (ret) {
-        /* lookup_host already set the error string. */
-        ne_close_connection(sess);
-        return ret;
-    }
-    
-    /* Find the first IPv4 address available for the server. */
-    for (ia = ne_addr_first(sess->server.address);
-         ia && ne_iaddr_typeof(ia) == ne_iaddr_ipv6;
-         ia = ne_addr_next(sess->server.address)) {
-        /* noop */
-    }
-
-    /* ... if any */
-    if (ia == NULL) {
-        ne_set_error(sess, _("Could not find IPv4 address of "
-                             "hostname %s for SOCKS v4 proxy"), 
-                     sess->server.hostname);
-        ne_close_connection(sess);
-        return NE_LOOKUP;
-    }
-
-    sess->server.current = ia;
-    
-    return ret;
-}
-
 static int open_connection(ne_session *sess) 
 {
     int ret;
+    struct host_info *host;
     
     if (sess->connected) return NE_OK;
 
-    if (!sess->proxies) {
-        ret = do_connect(sess, &sess->server);
-        if (ret) {
-            sess->nexthop = NULL;
-            return ret;
-        }
-    }
-    else {
-        struct host_info *hi;
-
-        /* Attempt to re-use proxy to avoid iterating through
-         * unnecessarily. */
-        if (sess->prev_proxy) 
-            ret = do_connect(sess, sess->prev_proxy);
-        else
-            ret = NE_ERROR;
-
-        /* Otherwise, try everything - but omitting prev_proxy if that
-         * has already been tried. */
-        for (hi = sess->proxies; hi && ret; hi = hi->next) {
-            if (hi != sess->prev_proxy)
-                ret = do_connect(sess, hi);
-        }
-
-        if (ret == NE_OK && sess->nexthop->proxy == PROXY_SOCKS) {
-            /* Special-case for SOCKS v4 proxies, which require the
-             * client to resolve the origin server IP address. */
-            if (sess->socks_ver == NE_SOCK_SOCKSV4) {
-                ret = socks_origin_lookup(sess);
-            }
-            
-            if (ret == NE_OK) {
-                /* Perform the SOCKS handshake, instructing the proxy
-                 * to set up the connection to the origin server. */
-                ret = ne_sock_proxy(sess->socket, sess->socks_ver, 
-                                    sess->server.current,
-                                    sess->server.hostname, sess->server.port,
-                                    sess->socks_user, sess->socks_password);
-                if (ret) {
-                    ne_set_error(sess, 
-                                 _("Could not establish connection from "
-                                   "SOCKS proxy (%s:%u): %s"),
-                                 sess->nexthop->hostname,
-                                 sess->nexthop->port,
-                                 ne_sock_error(sess->socket));
-                    ne_close_connection(sess);
-                    ret = NE_ERROR;
-                }
-            }
-        }
-
-        if (ret != NE_OK) {
-            sess->nexthop = NULL;
-            sess->prev_proxy = NULL;
-            return ret;
-        }
-        
-        /* Success - make this proxy stick. */
-        sess->prev_proxy = hi;
-    }
+    /* Resolve hostname if necessary. */
+    host = sess->use_proxy ? &sess->proxy : &sess->server;
+    if (host->address == NULL) {
+        ret = lookup_host(sess, host);
+        if (ret) return ret;
+    }    
+    
+    ret = do_connect(sess, host, 
+                     sess->use_proxy ? 
+                     _("Could not connect to proxy server")
+                     : _("Could not connect to server"));
+    if (ret != NE_OK) return ret;
 
 #ifdef NE_HAVE_SSL
     /* Negotiate SSL layer if required. */
     if (sess->use_ssl && !sess->in_connect) {
-        /* Set up CONNECT tunnel if using an HTTP proxy. */
-        if (sess->nexthop->proxy == PROXY_HTTP)
+        /* CONNECT tunnel */
+        if (sess->use_proxy)
             ret = proxy_tunnel(sess);
         
         if (ret == NE_OK) {
