@@ -1,6 +1,6 @@
 /* 
    HTTP session handling
-   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2007, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -61,6 +61,15 @@ typedef enum ne_session_flag_e {
     NE_SESSFLAG_SSLv2, /* disable this flag to disable support for
                         * SSLv2, if supported by the SSL library. */
 
+    NE_SESSFLAG_RFC4918, /* enable this flag to enable support for
+                          * RFC4918-only WebDAV features; losing
+                          * backwards-compatibility with RFC2518
+                          * servers. */
+
+    NE_SESSFLAG_CONNAUTH, /* enable this flag if an awful, broken,
+                           * RFC-violating, connection-based HTTP
+                           * authentication scheme is in use. */
+
     NE_SESSFLAG_LAST /* enum sentinel value */
 } ne_session_flag;
 
@@ -76,34 +85,80 @@ int ne_get_session_flag(ne_session *sess, ne_session_flag flag);
  * must remain valid until the session is destroyed. */
 void ne_set_addrlist(ne_session *sess, const ne_inet_addr **addrs, size_t n);
 
-/* Progress callback. */
-typedef void (*ne_progress)(void *userdata, off_t progress, off_t total);
+/* DEPRECATED: Progress callback. */
+typedef void (*ne_progress)(void *userdata, ne_off_t progress, ne_off_t total);
 
-/* Set a progress callback for the session. */
-void ne_set_progress(ne_session *sess, 
-		     ne_progress progress, void *userdata);
+/* DEPRECATED API: Set a progress callback for the session; this is
+ * deprecated in favour of ne_set_notifier().  The progress callback
+ * is invoked for after each block of the request and response body to
+ * indicate request and response progress (there is no way to
+ * distinguish between the two using this interface alone).
+ *
+ * NOTE: Use of this interface is mutually exclusive with the use of
+ * ne_set_notifier().  A call to ne_set_progress() removes the
+ * notifier callback, and vice versa. */
+void ne_set_progress(ne_session *sess, ne_progress progress, void *userdata);
 
 /* Store an opaque context for the session, 'priv' is returned by a
  * call to ne_session_get_private with the same ID. */
 void ne_set_session_private(ne_session *sess, const char *id, void *priv);
 void *ne_get_session_private(ne_session *sess, const char *id);
 
+/* Status event type.  NOTE: More event types may be added in
+ * subsequent releases, so callers must ignore unknown status types
+ * for forwards-compatibility.  */
 typedef enum {
-    ne_conn_namelookup, /* lookup up hostname (info = hostname) */
-    ne_conn_connecting, /* connecting to host (info = hostname) */
-    ne_conn_connected, /* connected to host (info = hostname) */
-    ne_conn_secure /* connection now secure (info = crypto level) */
-} ne_conn_status;
+    ne_status_lookup = 0, /* looking up hostname */
+    ne_status_connecting, /* connecting to host */
+    ne_status_connected, /* connected to host */
+    ne_status_sending, /* sending a request body */
+    ne_status_recving, /* receiving a response body */
+    ne_status_disconnected /* disconnected from host */
+} ne_session_status;
 
-typedef void (*ne_notify_status)(void *userdata, 
-				 ne_conn_status status,
-				 const char *info);
+/* Status event information union; the relevant structure within
+ * corresponds to the event type.  WARNING: the size of this union is
+ * not limited by ABI constraint; it may be extended with additional
+ * members of different size, or existing members may be extended. */
+typedef union ne_session_status_info_u {
+    struct /* ne_status_lookup */ {
+        /* The hostname which is being resolved: */
+        const char *hostname;
+    } lu;
+    struct /* ne_status_connecting */ {
+        /* The hostname and network address to which a connection
+         * attempt is being made: */
+        const char *hostname;
+        const ne_inet_addr *address;
+    } ci;
+    struct /* ne_status_connected, ne_status_disconnected */ {
+        /* The hostname to which a connection has just been
+         * established or closed: */
+        const char *hostname;
+    } cd;
+    struct /* ne_status_sending and ne_status_recving */ {
+        /* Request/response body transfer progress; if total == -1, the
+         * total size is unknown; else 0 <= progress <= total:  */
+        ne_off_t progress, total;
+    } sr;
+} ne_session_status_info;
 
+/* Callback invoked to notify a new session status event, given by the
+ * 'status' argument.  On invocation, the contents of exactly one of
+ * the structures in the info union will be valid, as indicated
+ * above. */
+typedef void (*ne_notify_status)(void *userdata, ne_session_status status,
+                                 const ne_session_status_info *info);
 
 /* Set a status notification callback for the session, to report
- * connection status. */
-void ne_set_status(ne_session *sess,
-		   ne_notify_status status, void *userdata);
+ * session status events.  Only one notification callback per session
+ * can be registered; the most recent of successive calls to this
+ * function takes effect. Note that 
+ *
+ * NOTE: Use of this interface is mutually exclusive with the use of
+ * ne_set_progress().  A call to ne_set_notifier() removes the
+ * progress callback, and vice versa. */
+void ne_set_notifier(ne_session *sess, ne_notify_status status, void *userdata);
 
 /* Certificate verification failures.
  * The certificate is not yet valid: */
@@ -145,7 +200,8 @@ void ne_ssl_set_verify(ne_session *sess, ne_ssl_verify_fn fn, void *userdata);
 void ne_ssl_set_clicert(ne_session *sess, const ne_ssl_client_cert *clicert);
 
 /* Indicate that the certificate 'cert' is trusted; the 'cert' object
- * is duplicated internally so can be destroyed by the caller. */
+ * is duplicated internally so can be destroyed by the caller.  This
+ * function has no effect for non-SSL sessions. */
 void ne_ssl_trust_cert(ne_session *sess, const ne_ssl_certificate *cert);
 
 /* If the SSL library provided a default set of CA certificates, trust
@@ -170,6 +226,10 @@ void ne_ssl_provide_clicert(ne_session *sess,
  * timeout value must be greater than zero. */
 void ne_set_read_timeout(ne_session *sess, int timeout);
 
+/* Set the timeout (in seconds) used when making a connection.  The
+ * timeout value must be greater than zero. */
+void ne_set_connect_timeout(ne_session *sess, int timeout);
+
 /* Sets the user-agent string. neon/VERSION will be appended, to make
  * the full header "User-Agent: product neon/VERSION".
  * If this function is not called, the User-Agent header is not sent.
@@ -191,10 +251,18 @@ const char *ne_get_server_hostport(ne_session *sess);
  * the trailing ':'; e.g. "http" or "https". */
 const char *ne_get_scheme(ne_session *sess);
 
-/* Sets the host, scheme, and port fields (and no others) of the given
- * URI structure; host and scheme are malloc-allocated. */
+/* Sets the host, scheme, and port fields of the given URI structure
+ * to that of the configured server and scheme for the session; host
+ * and scheme are malloc-allocated.  No other fields in the URI
+ * structure are changed. */
 void ne_fill_server_uri(ne_session *sess, ne_uri *uri);
 
+/* If a proxy is configured, sets the host and port fields in the
+ * given URI structure to that of the proxy.  The hostname is
+ * malloc-allocated.  No other fields in the URI structure are
+ * changed; if a proxy is not configured, no fields are changed. */
+void ne_fill_proxy_uri(ne_session *sess, ne_uri *uri);
+
 /* Set the error string for the session; takes printf-like format
  * string. */
 void ne_set_error(ne_session *sess, const char *format, ...)
