@@ -1,6 +1,6 @@
 /* 
    socket handling interface
-   Copyright (C) 1999-2010, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -24,10 +24,6 @@
 
 #include <sys/types.h>
 
-#ifdef WIN32
-#include <stdlib.h> /* for size_t */
-#endif
-
 #include "ne_defs.h"
 #include "ne_ssl.h" /* for ne_ssl_context */
 
@@ -42,7 +38,6 @@ NE_BEGIN_DECLS
 #define NE_SOCK_RESET (-4)
 /* Secure connection was closed without proper SSL shutdown. */
 #define NE_SOCK_TRUNC (-5)
-/* Reserved value -   (-6) */
 
 /* ne_socket represents a TCP socket. */
 typedef struct ne_socket_s ne_socket;
@@ -63,11 +58,9 @@ int ne_sock_init(void);
  * times to ne_sock_init() for the process. */
 void ne_sock_exit(void);
 
-#define NE_ADDR_CANON (0x01)
-/* Resolve the given hostname. Hex string IPv6 addresses (e.g. `::1')
- * may be enclosed in brackets (e.g. `[::1]').  'flags' should be
- * zero, or if NE_ADDR_CANON is passed, the canonical name for the
- * hostname will be determind. */
+/* Resolve the given hostname.  'flags' must be zero.  Hex
+ * string IPv6 addresses (e.g. `::1') may be enclosed in brackets
+ * (e.g. `[::1]'). */
 ne_sock_addr *ne_addr_resolve(const char *hostname, int flags);
 
 /* Returns zero if name resolution was successful, non-zero on
@@ -91,11 +84,6 @@ const ne_inet_addr *ne_addr_next(ne_sock_addr *addr);
  * which is of size 'bufsiz'.  'buffer' is returned. */
 char *ne_addr_error(const ne_sock_addr *addr, char *buffer, size_t bufsiz);
 
-/* Returns the canonical name of the host as a NUL-terminated string,
- * if NE_ADDR_CANON was used, and name resolution was successful.
- * Otherwise, returns NULL. */
-const char *ne_addr_canonical(const ne_sock_addr *addr);
-
 /* Destroys an address object created by ne_addr_resolve. */
 void ne_addr_destroy(ne_sock_addr *addr);
 
@@ -122,41 +110,17 @@ ne_iaddr_type ne_iaddr_typeof(const ne_inet_addr *ia);
  * buffer 'buffer', which is of length 'bufsiz'.  Returns 'buffer'. */
 char *ne_iaddr_print(const ne_inet_addr *ia, char *buffer, size_t bufsiz);
 
-/* Dump the raw byte representation (in network byte order) of address
- * 'ia' into the buffer 'buffer', which must be of a suitable length
- * (4 bytes for an IPv4 address, 16 bytes for an IPv6 address).
- * Returns 'buffer'. */
-unsigned char *ne_iaddr_raw(const ne_inet_addr *ia, unsigned char *buffer);
-
 /* Perform the reverse name lookup on network address 'ia', placing
  * the returned name in the 'buf' buffer (of length 'bufsiz') if
  * successful.  Returns zero on success, or non-zero on error. */
 int ne_iaddr_reverse(const ne_inet_addr *ia, char *buf, size_t bufsiz);
 
-/* Convert network address string 'addr' (for example, "127.0.0.1")
- * into a network address object.  Returns NULL on parse error.  If
- * non-NULL, return value must be freed using ne_iaddr_free. */
-ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type);
-
-/* Destroy a network address object created using ne_iaddr_make or
- * ne_iaddr_parse. */
+/* Destroy a network address object created using ne_iaddr_make. */
 void ne_iaddr_free(ne_inet_addr *addr);
 
 /* Create a socket object; returns NULL on error. */
 ne_socket *ne_sock_create(void);
 
-/* Specify an address to which the local end of the socket will be
- * bound during a subsequent ne_sock_connect() call.  If the address
- * passed to ne_sock_connect() is of a different type (family) to
- * 'addr', 'addr' is ignored.  Either 'addr' may be NULL, to use the
- * given port with unspecified address, or 'port' may be 0, to use the
- * given address with an unspecified port.
- *
- * (Note: This function is not equivalent to a BSD socket bind(), it
- * only takes effect during the _connect() call). */
-void ne_sock_prebind(ne_socket *sock, const ne_inet_addr *addr,
-                     unsigned int port);
-
 /* Connect the socket to server at address 'addr' on port 'port'.
  * Returns zero on success, NE_SOCK_TIMEOUT if a timeout occurs when a
  * non-zero connect timeout is configured (and is supported), or
@@ -191,19 +155,6 @@ int ne_sock_block(ne_socket *sock, int n);
  * on error. */
 int ne_sock_fullwrite(ne_socket *sock, const char *data, size_t count); 
 
-/* I/O vector. */
-struct ne_iovec {
-    void *base;
-    size_t len;
-};
-
-/* Writes 'count' blocks described by 'vector' to the socket.
- * Guarantees to either write all the bytes or to fail.  Count must be
- * greater than zero and smaller than the system-defined maximum
- * vector limit.  Returns 0 on success, or NE_SOCK_* on error. */
-int ne_sock_fullwritev(ne_socket *sock, const struct ne_iovec *vector,
-                       int count); 
-
 /* Read an LF-terminated line into 'buffer', and NUL-terminate it.
  * At most 'len' bytes are read (including the NUL terminator).
  * Returns:
@@ -223,24 +174,13 @@ int ne_sock_accept(ne_socket *sock, int fd);
 /* Returns the file descriptor used for socket 'sock'. */
 int ne_sock_fd(const ne_socket *sock);
 
-/* Return address of peer, or NULL on error.  The returned address
- * must be destroyed by caller using ne_iaddr_free. */
-ne_inet_addr *ne_sock_peer(ne_socket *sock, unsigned int *port);
-
-/* Close the socket and destroy the socket object.  If SSL is in use
- * for the socket, a closure alert is sent to initiate a clean
- * shutdown, but this function does not wait for the peer's response.
- * Returns zero on success, or non-zero on failure. */
+/* Close the socket and destroy the socket object.  Returns zero on
+ * success, or an errno value if close() failed. */
 int ne_sock_close(ne_socket *sock);
 
 /* Return current error string for socket. */
 const char *ne_sock_error(const ne_socket *sock);
 
-/* Set the error string for the socket; takes printf-like format
- * string. */
-void ne_sock_set_error(ne_socket *sock, const char *format, ...)
-    ne_attribute((format (printf, 2, 3)));
-
 /* Set read timeout for socket, in seconds; must be a non-zero
  * positive integer. */
 void ne_sock_read_timeout(ne_socket *sock, int timeout);
@@ -269,44 +209,6 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx,
  * Returns zero on success, non-zero on error. */
 int ne_sock_sessid(ne_socket *sock, unsigned char *buf, size_t *buflen);
 
-/* Return human-readable name of SSL/TLS cipher used for connection,
- * or NULL if none.  The format of this string is not intended to be
- * fixed or parseable, but is informational only.  Return value is
- * NUL-terminated malloc-allocated string if not NULL, which must be
- * freed by the caller. */
-char *ne_sock_cipher(ne_socket *sock);
-
-/* SOCKS proxy protocol version: */
-enum ne_sock_sversion {
-    NE_SOCK_SOCKSV4 = 0,
-    NE_SOCK_SOCKSV4A,
-    NE_SOCK_SOCKSV5
-};
-
-/* Given a socket 'sock' which is connected to a SOCKS proxy, initiate
- * a connection to a destination server using that proxy, specified
- * either by network address or hostname, at given port 'port'.
- *
- * If 'vers' is NE_SOCKS_V4, addr must be an IPv4 address; hostname
- * and password are ignored; username must be non-NULL.
- *
- * If 'vers' is NE_SOCKS_V4A, hostname must be non-NULL; addr is
- * ignored; password is ignored; username must be non-NULL.
- *
- * If 'vers' is NE_SOCKS_V5, addr may be NULL, in which case hostname
- * must be non-NULL.  addr if non-NULL may be an IPv4 or IPv6 address;
- * username may be NULL, in which case password is ignored.  If
- * username is non-NULL password must also be non-NULL.
- *
- * Returns 0 on success, or NE_SOCK_* on failure - in which case, the
- * socket error string is set.  On failure, the socket must be closed
- * by the caller.
- */
-int ne_sock_proxy(ne_socket *sock, enum ne_sock_sversion vers,
-                  const ne_inet_addr *addr, const char *hostname, 
-                  unsigned int port,
-                  const char *username, const char *password);
-
 NE_END_DECLS
 
 #endif /* NE_SOCKET_H */
