@@ -1,6 +1,6 @@
 /* 
    Authentication tests
-   Copyright (C) 2001-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2001-2007, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -373,13 +373,18 @@ struct digest_parms {
     int send_ainfo;
     int md5_sess;
     int proxy;
+    int send_nextnonce;
+    int num_requests;
+    int stale;
     enum digest_failure {
         fail_not,
         fail_bogus_alg,
+        fail_req0_stale,
         fail_omit_qop,
         fail_omit_realm,
         fail_omit_nonce,
         fail_ai_bad_nc,
+        fail_ai_bad_nc_syntax,
         fail_ai_bad_digest,
         fail_ai_bad_cnonce,
         fail_ai_omit_cnonce,
@@ -557,6 +562,8 @@ static int verify_digest_header(struct digest_state *state,
     ONN("no digest param given", !state->digest);
 
     CALL(check_digest(state, parms));
+
+    state->nc++;
     
     return OK;
 }
@@ -573,7 +580,9 @@ static char *make_authinfo_header(struct digest_state *state,
         make_digest(state, parms, 1, digest);
     }
 
-    if (parms->failure == fail_ai_bad_nc) {
+    if (parms->failure == fail_ai_bad_nc_syntax) {
+        ncval = "zztop";
+    } else if (parms->failure == fail_ai_bad_nc) {
         ncval = "999";
     } else {
         ncval = state->ncval;
@@ -603,6 +612,11 @@ static char *make_authinfo_header(struct digest_state *state,
         if (parms->failure != fail_ai_omit_digest) {
             ne_buffer_concat(buf, "rspauth=\"", digest, "\", ", NULL);
         }
+        if (parms->send_nextnonce) {
+            state->nonce = ne_concat("next-", state->nonce, NULL);
+            ne_buffer_concat(buf, "nextnonce=\"", state->nonce, "\", ", NULL);
+            state->nc = 1;
+        }
         ne_buffer_czappend(buf, "qop=\"auth\"");
     }
     
@@ -633,6 +647,11 @@ static char *make_digest_header(struct digest_state *state,
         ne_buffer_concat(buf, "opaque=\"", parms->opaque, "\", ", NULL);
     }
 
+    if (parms->failure == fail_req0_stale
+        || parms->stale == parms->num_requests) {
+        ne_buffer_concat(buf, "stale='true', ", NULL);
+    }
+
     ne_buffer_concat(buf, "nonce=\"", state->nonce, "\"", NULL);
 
     return ne_buffer_finish(buf);
@@ -658,6 +677,11 @@ static int serve_digest(ne_socket *sock, void *userdata)
 
     state.cnonce = state.digest = state.ncval = NULL;
 
+    parms->num_requests += parms->stale ? 1 : 0;
+
+    NE_DEBUG(NE_DBG_HTTP, ">>>> Response sequence begins, %d requests.\n",
+             parms->num_requests);
+
     want_header = parms->proxy ? "Proxy-Authorization" : "Authorization";
     digest_hdr = NULL;
     got_header = dup_header;
@@ -676,31 +700,45 @@ static int serve_digest(ne_socket *sock, void *userdata)
 
     SEND_STRING(sock, resp);
 
-    CALL(discard_request(sock));
+    do {
+        CALL(discard_request(sock));
 
-    ONN("no Authorization header sent", digest_hdr == NULL);
-
-    CALL(verify_digest_header(&state, parms, digest_hdr));
-
-    if (parms->send_ainfo) {
-        char *ai = make_authinfo_header(&state, parms);
-
-        ne_snprintf(resp, sizeof resp,
-                    "HTTP/1.1 200 Well, if you insist\r\n"
-                    "Content-Length: 0\r\n"
-                    "%s\r\n"
-                    "\r\n", ai);
+        ONN("no Authorization header sent", digest_hdr == NULL);
         
-        ne_free(ai);
+        CALL(verify_digest_header(&state, parms, digest_hdr));
 
-                    
-    } else {
-        ne_snprintf(resp, sizeof resp,
-                    "HTTP/1.1 200 You did good\r\n"
-                    "Content-Length: 0\r\n" "\r\n");
-    }
+        if (parms->num_requests == parms->stale) {
+            state.nonce = ne_concat("stale-", state.nonce, NULL);
+            state.nc = 1;
 
-    SEND_STRING(sock, resp);
+            ne_snprintf(resp, sizeof resp,
+                        "HTTP/1.1 %d Auth Denied\r\n"
+                        "%s\r\n"
+                        "Content-Length: 0\r\n" "\r\n",
+                        parms->proxy ? 407 : 401,
+                        make_digest_header(&state, parms));
+        }
+        else if (parms->send_ainfo) {
+            char *ai = make_authinfo_header(&state, parms);
+            
+            ne_snprintf(resp, sizeof resp,
+                        "HTTP/1.1 200 Well, if you insist\r\n"
+                        "Content-Length: 0\r\n"
+                        "%s\r\n"
+                        "\r\n", ai);
+            
+            ne_free(ai);
+        } else {
+            ne_snprintf(resp, sizeof resp,
+                        "HTTP/1.1 200 You did good\r\n"
+                        "Content-Length: 0\r\n" "\r\n");
+        }
+
+        SEND_STRING(sock, resp);
+
+        NE_DEBUG(NE_DBG_HTTP, "Handled request; %d requests remain.\n",
+                 parms->num_requests - 1);
+    } while (--parms->num_requests);
 
     return OK;
 }
@@ -709,6 +747,8 @@ static int test_digest(struct digest_parms *parms)
 {
     ne_session *sess;
 
+    NE_DEBUG(NE_DBG_HTTP, ">>>> Request sequence begins.\n");
+
     if (parms->proxy) {
         sess = ne_session_create("http", "www.example.com", 80);
         ne_session_proxy(sess, "localhost", 7777);
@@ -721,7 +761,9 @@ static int test_digest(struct digest_parms *parms)
 
     CALL(spawn_server(7777, serve_digest, parms));
 
-    CALL(any_2xx_request(sess, "/fish"));
+    do {
+        CALL(any_2xx_request(sess, "/fish"));
+    } while (--parms->num_requests);
     
     ne_session_destroy(sess);
     return await_server();
@@ -732,22 +774,27 @@ static int digest(void)
 {
     struct digest_parms parms[] = {
         /* RFC 2617-style */
-        { "WallyWorld", "this-is-a-nonce", NULL, 1, 0, 0, 0, fail_not },
-        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 0, 0, 0, fail_not },
+        { "WallyWorld", "this-is-a-nonce", NULL, 1, 0, 0, 0, 0, 1, 0, fail_not },
+        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 0, 0, 0, 0, 1, 0, fail_not },
         /* ... with A-I */
-        { "WallyWorld", "nonce-nonce-nonce", "opaque-string", 1, 1, 0, 0, fail_not },
+        { "WallyWorld", "nonce-nonce-nonce", "opaque-string", 1, 1, 0, 0, 0, 1, 0, fail_not },
         /* ... with md5-sess. */
-        { "WallyWorld", "nonce-nonce-nonce", "opaque-string", 1, 1, 1, 0, fail_not },
+        { "WallyWorld", "nonce-nonce-nonce", "opaque-string", 1, 1, 1, 0, 0, 1, 0, fail_not },
+        /* many requests, with changing nonces; tests for next-nonce handling bug. */
+        { "WallyWorld", "this-is-a-nonce", "opaque-thingy", 1, 1, 0, 0, 1, 20, 0, fail_not },
+        /* staleness. */
+        { "WallyWorld", "this-is-a-nonce", "opaque-thingy", 1, 1, 0, 0, 0, 3, 2, fail_not },
+
 
         /* RFC 2069-style */ 
-        { "WallyWorld", "lah-di-da-di-dah", NULL, 0, 0, 0, 0, fail_not },
-        { "WallyWorld", "fee-fi-fo-fum", "opaque-string", 0, 0, 0, 0, fail_not },
-        { "WallyWorld", "fee-fi-fo-fum", "opaque-string", 0, 1, 0, 0, fail_not },
+        { "WallyWorld", "lah-di-da-di-dah", NULL, 0, 0, 0, 0, 0, 1, 0, fail_not },
+        { "WallyWorld", "fee-fi-fo-fum", "opaque-string", 0, 0, 0, 0, 0, 1, 0, fail_not },
+        { "WallyWorld", "fee-fi-fo-fum", "opaque-string", 0, 1, 0, 0, 0, 1, 0, fail_not },
 
         /* Proxy auth */
-        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 1, 0, 0, fail_not },
+        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 1, 0, 0, 0, 1, 0, fail_not },
         /* Proxy + A-I */
-        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 1, 0, 1, fail_not },
+        { "WallyWorld", "this-is-also-a-nonce", "opaque-string", 1, 1, 0, 1, 0, 1, 0, fail_not },
 
         { NULL }
     };
@@ -769,12 +816,14 @@ static int digest_failures(void)
         const char *message;
     } fails[] = {
         { fail_ai_bad_nc, "nonce count mismatch" },
+        { fail_ai_bad_nc_syntax, "could not parse nonce count" },
         { fail_ai_bad_digest, "digest mismatch" },
         { fail_ai_bad_cnonce, "client nonce mismatch" },
         { fail_ai_omit_nc, "missing parameters" },
         { fail_ai_omit_digest, "missing parameters" },
         { fail_ai_omit_cnonce, "missing parameters" },
-        { fail_bogus_alg, "Unknown algorithm" },
+        { fail_bogus_alg, "unknown algorithm" },
+        { fail_req0_stale, "initial Digest challenge was stale" },
         { fail_not, NULL }
     };
     size_t n;
@@ -786,6 +835,9 @@ static int digest_failures(void)
     parms.send_ainfo = 1;
     parms.md5_sess = 0;
     parms.proxy = 0;
+    parms.send_nextnonce = 0;
+    parms.num_requests = 1;
+    parms.stale = 0;
 
     for (n = 0; fails[n].message; n++) {
         ne_session *sess = ne_session_create("http", "localhost", 7777);
@@ -807,7 +859,8 @@ static int digest_failures(void)
 
         ne_session_destroy(sess);
         
-        if (fails[n].mode == fail_bogus_alg) {
+        if (fails[n].mode == fail_bogus_alg
+            || fails[n].mode == fail_req0_stale) {
             reap_server();
         } else {
             CALL(await_server());
@@ -817,8 +870,144 @@ static int digest_failures(void)
     return OK;
 }
 
-/* test that digest has precedence over Basic for multi-scheme
- * challenges */
+static int fail_cb(void *userdata, const char *realm, int tries, 
+		   char *un, char *pw)
+{
+    ne_buffer *buf = userdata;
+    char str[64];
+
+    ne_snprintf(str, sizeof str, "<%s, %d>", realm, tries);
+    ne_buffer_zappend(buf, str);
+
+    return -1;
+}
+
+static int fail_challenge(void)
+{
+    static const struct {
+        const char *resp, *error, *challs;
+    } ts[] = {
+        /* only possible Basic parse failure. */
+        { "Basic", "missing realm in Basic challenge" },
+
+        /* Digest parameter invalid/omitted failure cases: */
+        { "Digest algorithm=MD5, qop=auth, nonce=\"foo\"",
+          "missing parameter in Digest challenge" },
+        { "Digest algorithm=MD5, qop=auth, realm=\"foo\"",
+          "missing parameter in Digest challenge" },
+        { "Digest algorithm=ZEBEDEE-GOES-BOING, qop=auth, realm=\"foo\"",
+          "unknown algorithm in Digest challenge" },
+        { "Digest algorithm=MD5-sess, realm=\"foo\"",
+          "incompatible algorithm in Digest challenge" },
+
+        /* Multiple challenge failure cases: */
+        { "Basic, Digest",
+          "missing parameter in Digest challenge, missing realm in Basic challenge" },
+        
+        { "Digest realm=\"foo\", algorithm=MD5, qop=auth, nonce=\"foo\","
+          " Basic realm=\"foo\"",
+          "rejected Digest challenge, rejected Basic challenge" },
+
+        { "WhizzBangAuth realm=\"foo\", " 
+          "Basic realm='foo'",
+          "ignored WhizzBangAuth challenge, rejected Basic challenge" },
+        { "", "could not parse challenge" },
+
+        /* neon 0.26.x regression in "attempt" handling. */
+        { "Basic realm=\"foo\", " 
+          "Digest realm=\"bar\", algorithm=MD5, qop=auth, nonce=\"foo\"",
+          "rejected Digest challenge, rejected Basic challenge"
+          , "<bar, 0><foo, 1>"  /* Digest challenge first, Basic second. */
+        }
+    };
+    unsigned n;
+    
+    for (n = 0; n < sizeof(ts)/sizeof(ts[0]); n++) {
+        char resp[512];
+        ne_session *sess;
+        int ret;
+        ne_buffer *buf = ne_buffer_create();
+
+        ne_snprintf(resp, sizeof resp,
+                    "HTTP/1.1 401 Auth Denied\r\n"
+                    "WWW-Authenticate: %s\r\n"
+                    "Content-Length: 0\r\n" "\r\n",
+                    ts[n].resp);
+        
+        CALL(make_session(&sess, single_serve_string, resp));
+
+        ne_set_server_auth(sess, fail_cb, buf);
+        
+        ret = any_2xx_request(sess, "/fish");
+        ONV(ret == NE_OK,
+            ("request success; expecting error '%s'",
+             ts[n].error));
+
+        ONV(strstr(ne_get_error(sess), ts[n].error) == NULL,
+            ("request fails with error '%s'; expecting '%s'",
+             ne_get_error(sess), ts[n].error));
+        
+        if (ts[n].challs) {
+            ONCMP(ts[n].challs, buf->data, "challenge callback", 
+                  "invocation order");
+        }
+
+        ne_session_destroy(sess);
+        ne_buffer_destroy(buf);
+        CALL(await_server());
+    }
+
+    return OK;
+}
+
+struct multi_context {
+    int id;
+    ne_buffer *buf;
+};
+
+static int multi_cb(void *userdata, const char *realm, int tries, 
+                    char *un, char *pw)
+{
+    struct multi_context *ctx = userdata;
+
+    ne_buffer_snprintf(ctx->buf, 128, "[id=%d, realm=%s, tries=%d]", 
+                       ctx->id, realm, tries);
+
+    return -1;
+}
+
+static int multi_handler(void)
+{
+    ne_session *sess;
+    struct multi_context c[2];
+    unsigned n;
+    ne_buffer *buf = ne_buffer_create();
+
+    CALL(make_session(&sess, single_serve_string,
+                      "HTTP/1.1 401 Auth Denied\r\n"
+                      "WWW-Authenticate: Basic realm='fish',"
+                      " Digest realm='food', algorithm=MD5, qop=auth, nonce=gaga\r\n"
+                      "Content-Length: 0\r\n" "\r\n"));
+    
+    for (n = 0; n < 2; n++) {
+        c[n].buf = buf;
+        c[n].id = n + 1;
+    }
+
+    ne_add_server_auth(sess, NE_AUTH_BASIC, multi_cb, &c[0]);
+    ne_add_server_auth(sess, NE_AUTH_DIGEST, multi_cb, &c[1]);
+    
+    any_request(sess, "/fish");
+    
+    ONCMP("[id=2, realm=food, tries=0]"
+          "[id=1, realm=fish, tries=0]", buf->data,
+          "multiple callback", "invocation order");
+    
+    ne_session_destroy(sess);
+    ne_buffer_destroy(buf);
+
+    return await_server();
+}
 
 /* test logout */
 
@@ -833,5 +1022,7 @@ ne_test tests[] = {
     T(negotiate_regress),
     T(digest),
     T(digest_failures),
+    T(fail_challenge),
+    T(multi_handler),
     T(NULL)
 };
