@@ -1,6 +1,6 @@
 /* 
    HTTP request handling tests
-   Copyright (C) 2001-2008, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2001-2007, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -203,7 +203,7 @@ static int reason_phrase(void)
 
     CALL(make_session(&sess, single_serve_string, RESP200
 		      "Connection: close\r\n\r\n"));
-    ONREQ(any_request(sess, "/foo"));
+    CALL(any_request(sess, "/foo"));
     CALL(await_server());
     
     ONV(strcmp(ne_get_error(sess), "200 OK"),
@@ -1607,12 +1607,12 @@ static int versions(void)
 		      "HTTP/1.0 200 OK\r\n"
 		      "Content-Length: 0\r\n\r\n"));
     
-    ONREQ(any_request(sess, "/http11"));
+    CALL(any_request(sess, "/http11"));
 	 
     ONN("did not detect HTTP/1.1 compliance",
 	ne_version_pre_http11(sess) != 0);
 	 
-    ONREQ(any_request(sess, "/http10"));
+    CALL(any_request(sess, "/http10"));
 
     ONN("did not detect lack of HTTP/1.1 compliance",
 	ne_version_pre_http11(sess) == 0);
@@ -1655,7 +1655,7 @@ static int hook_create_req(void)
     args.uri = "/foo";
     args.result = -1;
 
-    ONREQ(any_request(sess, "/foo"));
+    CALL(any_request(sess, "/foo"));
     
     ONN("first hook never called", args.result == -1);
     if (args.result) return FAIL;
@@ -1666,7 +1666,7 @@ static int hook_create_req(void)
     /* force use of absoluteURI in request-uri */
     ne_session_proxy(sess, "localhost", 7777);
 
-    ONREQ(any_request(sess, "/bar"));
+    CALL(any_request(sess, "/bar"));
     
     ONN("second hook never called", args.result == -1);
     if (args.result) return FAIL;
@@ -1757,22 +1757,24 @@ static int send_bad_offset(void)
 {
     ne_session *sess;
     ne_request *req;
-    int ret, fds[2];
+    char *fn = "empty.txt";
+    int ret, fd;
 
     CALL(make_session(&sess, single_serve_string,
                       RESP200 "Content-Length: 0\r\n" "\r\n"));
 
-    /* create a pipe, on which seek is guaranteed to fail. */
-    ONN("could not create pipe", pipe(fds) != 0);
+    fd = open(fn, O_RDWR|O_CREAT);
+
+    ONV(fd < 0, ("could not find %s", fn));
 
     req = ne_request_create(sess, "PUT", "/null");
 
-    ne_set_request_body_fd(req, fds[0], 500, 5);
+    ne_set_request_body_fd(req, fd, -500, 5);
     
     ret = ne_request_dispatch(req);
 
-    close(fds[0]);
-    close(fds[1]);
+    close(fd);
+    unlink(fn);
 
     ONN("request dispatched with bad offset!", ret == NE_OK);
     ONV(ret != NE_ERROR,
@@ -1851,13 +1853,6 @@ static void hook_destroy_sess(void *userdata)
     ne_buffer_czappend(buf, "(destroy-sess)\n");
 }
 
-static void hook_close_conn(void *userdata)
-{
-    ne_buffer *buf = userdata;
-
-    ne_buffer_czappend(buf, "(close-conn)\n");
-}
-
 static int hooks(void)
 {
     ne_buffer *buf = ne_buffer_create();
@@ -1875,7 +1870,6 @@ static int hooks(void)
     ne_hook_post_send(sess, hook_post_send, buf);
     ne_hook_destroy_request(sess, hook_destroy_req, buf);
     ne_hook_destroy_session(sess, hook_destroy_sess, buf);
-    ne_hook_close_conn(sess, hook_close_conn, buf);
 
     CALL(any_2xx_request(sess, "/first"));
 
@@ -1921,8 +1915,7 @@ static int hooks(void)
     ne_session_destroy(sess);
     CALL(await_server());
 
-    ONCMP("(destroy-sess)\n"
-          "(close-conn)\n", buf->data, "hook ordering", "first destroyed session");
+    ONCMP("(destroy-sess)\n", buf->data, "hook ordering", "first destroyed session");
 
     ne_buffer_clear(buf);
 
@@ -2058,7 +2051,7 @@ static int status_chunked(void)
     ne_session_destroy(sess);
     CALL(await_server());
     
-    /* This sequence is not guaranteed by the API, but it's what the
+    /* This sequence is not guaranted by the API, but it's what the
      * current implementation should do. */
     ONCMP("lookup(localhost)-"
           "connecting(localhost,127.0.0.1)-"
@@ -2079,26 +2072,6 @@ static int status_chunked(void)
     return OK;
 }
 
-static const unsigned char raw_127[4] = "\x7f\0\0\01"; /* 127.0.0.1 */
-
-static int local_addr(void)
-{
-    ne_session *sess;
-    ne_inet_addr *ia = ne_iaddr_make(ne_iaddr_ipv4, raw_127);
-
-    CALL(make_session(&sess, single_serve_string, RESP200 
-                      "Connection: close\r\n\r\n"));
-
-    ne_set_localaddr(sess, ia);
-
-    ONREQ(any_request(sess, "/foo"));
-
-    ne_session_destroy(sess);
-    ne_iaddr_free(ia);
-
-    return reap_server();
-}
-
 ne_test tests[] = {
     T(lookup_localhost),
     T(single_get_clength),
@@ -2184,6 +2157,5 @@ ne_test tests[] = {
     T(icy_protocol),
     T(status),
     T(status_chunked),
-    T(local_addr),
     T(NULL)
 };
