@@ -502,6 +502,11 @@ static int ipaddr_altname(void)
     return accept_signed_cert_for_hostname("altname5.cert", "127.0.0.1");
 }
 
+static int uri_altname(void)
+{
+    return accept_signed_cert_for_hostname("altname7.cert", "localhost");
+}
+
 /* test that the *most specific* commonName attribute is used. */
 static int multi_commonName(void)
 {
@@ -791,6 +796,12 @@ static int fail_host_ipaltname(void)
                             "bad IP altname cert", NE_SSL_IDMISMATCH);
 }
 
+static int fail_bad_urialtname(void)
+{
+    return fail_ssl_request("altname8.cert", CA_CERT, "localhost",
+                            "bad URI altname cert", NE_SSL_IDMISMATCH);
+}
+
 /* Test that the SSL session is cached across connections. */
 static int session_cache(void)
 {
@@ -1112,7 +1123,7 @@ static int auth_tunnel_fail(void)
     ret = any_request(sess, "/bar");
     ONV(ret != NE_PROXYAUTH, ("bad error code for tunnel failure: %d", ret));
 
-    ONV(strstr(ne_get_error(sess), "Nyaaaaah") == NULL,
+    ONV(strstr(ne_get_error(sess), "GaBoogle") == NULL,
         ("bad error string for tunnel failure: %s", ne_get_error(sess)));
 
     ne_session_destroy(sess);
@@ -1180,6 +1191,7 @@ static int cert_identities(void)
         { "altname2.cert", "nohost.example.com" },
         { "altname4.cert", "localhost" },
         { "ca4.pem", "fourth.example.com" },
+        { "altname8.cert", "http://nohost.example.com/" },
         { NULL, NULL }
     };
     int n;
@@ -1479,6 +1491,17 @@ static int cache_cert(void)
     return OK;
 }
 
+static int nonssl_trust(void)
+{
+    ne_session *sess = ne_session_create("http", "www.example.com", 80);
+    
+    ne_ssl_trust_cert(sess, def_ca_cert);
+    
+    ne_session_destroy(sess);
+
+    return OK;
+}
+
 /* TODO: code paths still to test in cert verification:
  * - server cert changes between connections: Mozilla gives
  * a "bad MAC decode" error for this; can do better?
@@ -1535,6 +1558,7 @@ ne_test tests[] = {
     T(two_subject_altname2),
     T(notdns_altname),
     T(ipaddr_altname),
+    T(uri_altname),
 
     T(multi_commonName),
     T(commonName_first),
@@ -1547,6 +1571,7 @@ ne_test tests[] = {
     T(fail_missing_CN),
     T(fail_host_ipaltname),
     T(fail_bad_ipaltname),
+    T(fail_bad_urialtname),
 
     T(session_cache),
 	
@@ -1556,5 +1581,7 @@ ne_test tests[] = {
     T(auth_tunnel_creds),
     T(auth_tunnel_fail),
 
+    T(nonssl_trust),
+
     T(NULL) 
 };
